// app.js

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'hh' is the name of this angular module (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'

'use strict';

(function main(angular, app) {

window.hh = angular.module('hh', ['ionic', 'ngIOS9UIWebViewPatch', 'ngCordova', 'hh.controllers', 'hh.UserService', 'hh.LocalStorageService', 'ionic.contrib.drawer'])

.constant('AppConfig', app.configs.env)

.run([
	'$ionicHistory',
	'$ionicPlatform',
	'$rootScope',
	'$state',
	'$timeout',
	'AppConfig',
	'LocalStorageService',
	'PushService',
	'UserService',
	function(
		$ionicHistory,
		$ionicPlatform,
		$rootScope,
		$state,
		$timeout,
		AppConfig,
		LocalStorageService,
		PushService,
		UserService
	) {

		$ionicPlatform.ready(function() {
			
			var oauthCredentials = {
					facebook: AppConfig.oauth.facebookKey,
			},
			oauthOptions = { oauth_proxy: AppConfig.oauth.proxyUrl };
			oauthOptions.scope = "public_profile,email,user_birthday,user_relationship_details,user_work_history";

			// initialize Local Storage, default to always use persistent storage
			LocalStorageService.init(true);

			if (AppConfig.isMobileApp) {
				oauthOptions.redirect_uri = AppConfig.oauth.redirectUrl;
			}
			else if (AppConfig.oauth.redirectWebUrl) {
				oauthOptions.redirect_uri = AppConfig.oauth.redirectWebUrl;
			}

			// init hello.js
			hello.init(oauthCredentials, oauthOptions);
			
			initEvents();
			
			initUserModel();
			
			ga.trackEvent('session', 'appLaunch');
		});


		function initEvents() {
			$ionicPlatform.on("resume", function(event) {
				console.log("app resume");
				if (_urlScheme) {
					console.log("_urlScheme", JSON.stringify(_urlScheme));
				}
				$rootScope.$emit("checkMeet");
     		});
		}
		
		
		function initUserModel() {
			var userModel = UserService.getLoggedInUser();
			console.log("app.js getLoggedInUser", JSON.stringify(userModel));
			if (userModel) {
				var userParam = { userId: userModel.userId, session: userModel.session };
				UserService.getUser(userParam).then(
					function(data) {
						console.log("app.js getUser", JSON.stringify(data));
						if (data && data.r && data.r == "e") {
							UserService.clearUserModel();
							goHome();
						}
						else {
							if (userModel) {
								userModel = angular.extend(userModel, data.data);
							}
							else {
								userModel = data.data;
							}
							$ionicHistory.nextViewOptions({
								disableBack: true
							});
							goChat();
							_isInitPush = true;
							PushService.init();
						}
					},
					function(error) {
						console.log("app.js getUser error");
						goHome();
					}
				);
			}
			else {
				console.log("no userModel");
				goHome();
			}
		}
		

		function goHome() {
			hideSplash();
			$timeout(function() { $state.go('home'); }, 100);
		}


		function goChat() {
			hideSplash();
			$timeout(function() { $state.go('chat'); }, 100);
		}
		
		
		function hideSplash() {
			// hide splash screen
			navigator.splashscreen.hide();
				
			// translucent status bar with white text
			if (window.StatusBar) {
				StatusBar.styleBlackTranslucent();
			}
		}
	}
])

.config([
	'$stateProvider',
	'$ionicConfigProvider',
	'$sceDelegateProvider',
	function(
		$stateProvider,
		$ionicConfigProvider,
		$sceDelegateProvider
	) {
	
	$ionicConfigProvider.views.swipeBackEnabled(false);

	$ionicConfigProvider.backButton.text('');
	
	$ionicConfigProvider.backButton.previousTitleText(false);
	
	$stateProvider.state('home', {
		url: '/home',
		controller: 'HomeController',
		templateUrl: 'views/home.html',
		cache: false
	})
	.state('chat', {
		url: '/chat',
		controller: 'ChatController',
		templateUrl: 'views/chat.html',
		cache: false
	})
	.state('chat/chatroom', {
		url: '/chat/:chatroom/:category/:activity',
		controller: 'ChatroomController',
		templateUrl: 'views/chatroom.html',
		cache: false
	})
	.state('profile', {
		url: '/profile',
		controller: 'ProfileController',
		templateUrl: 'views/profile-edit.html',
		cache: false
	});
	
	$sceDelegateProvider.resourceUrlWhitelist(['**']);
}]);

})(window.angular, window.app);
