angular.module('hh.controllers', [])
.controller('HomeController',
['$cordovaInAppBrowser', '$scope', '$state', '$timeout', 'AppConfig', 'FacebookService', 'UserService',
function($cordovaInAppBrowser, $scope, $state, $timeout, AppConfig, FacebookService, UserService) {

	_isInitPush = false;

	var joinTexts = {
		join: "Join with Facebook",
		joining: "Joining...",
		joined: "Welcome to JONES...",
		age_range: "JONES is for age 21+"
	};

	var userModel = UserService.getLoggedInUser();

	$scope.stateData =
		userModel
			? { text: joinTexts.joined,
		 		joining: false,
				joined: true }
			: { text: joinTexts.join,
				joining: false,
				joined: false };


	$scope.signIn = function(authType) {
		if (userModel) {
			goChat();
			return;
		}

		switch (authType) {
			case 'f':
				$scope.stateData.text = joinTexts.joining;
				$scope.stateData.joining = true;
				$timeout(
					function() {
						FacebookService.login().then(onLoginSuccess, onLoginError);
					},
				100);
				break;
		}
	};


	$scope.openTerms = function() {
		$cordovaInAppBrowser.open(AppConfig.urlTerms, 'jones');
	}


	$scope.openPrivacy = function() {
		$cordovaInAppBrowser.open(AppConfig.urlPrivacy, 'jones');
	}


	function onLoginSuccess() {
		//console.log("HomeControllers onLoginSuccess");
		$scope.stateData.text = joinTexts.joined;
		$scope.stateData.joining = false;
		$scope.stateData.joined = true;
		userModel = UserService.getLoggedInUser();
		goChat();
	}


	function onLoginError(error) {
		console.log("HomeControllers onLoginError", error);
		if (error == "age_range") {
			$scope.stateData.text = joinTexts.age_range;
		}
		else {
			$scope.stateData.text = error;
		}
		$scope.stateData.joining = false;
	}


	function goChat() {
		$timeout(function() { $state.go('chat'); }, 100);
	}
}]);


angular.module('hh.controllers')
.controller('ProfileController', ['$ionicSlideBoxDelegate', '$ionicHistory', '$scope', '$state', '$timeout', 'AddMediaService', 'UserService', function($ionicSlideBoxDelegate, $ionicHistory, $scope, $state, $timeout, AddMediaService, UserService) {

	var userModel = UserService.getLoggedInUser();

	$scope.ages = [];
	for (var i = 18; i < 100; i++) {
		$scope.ages.push(i.toString());
	}

	$scope.done = function() {
		console.log("done:", JSON.stringify($scope.userModel));
		UserService.update($scope.userModel,
			function(data) {
				$timeout(function() { $state.go('chat'); }, 100);
			}
		);
	};

	$scope.setFemale = function() {
		if ($scope.userModel)
			$scope.userModel.gender = 'f';
		if ($scope.userModel.orientation)
			$scope.userModel.orientation = 'f' + $scope.userModel.orientation.charAt(1);
	};

	$scope.setMale = function() {
		if ($scope.userModel)
			$scope.userModel.gender = 'm';
		if ($scope.userModel.orientation)
			$scope.userModel.orientation = 'm' + $scope.userModel.orientation.charAt(1);
	};

	$scope.isFemale = function() {
		if ($scope.userModel && $scope.userModel.gender == 'f')
			return true;
		return false;
	};

	$scope.isMale = function() {
		if ($scope.userModel && $scope.userModel.gender == 'm')
			return true;
		return false;
	};

	$scope.setPrefersFemale = function() {
		if ($scope.userModel)
			$scope.userModel.orientation = $scope.userModel.gender + 'f';
	};

	$scope.setPrefersMale = function() {
		if ($scope.userModel)
			$scope.userModel.orientation = $scope.userModel.gender + 'm';
	};

	$scope.isPrefersFemale = function() {
		if ($scope.userModel && $scope.userModel.orientation && $scope.userModel.orientation.charAt(1) == 'f')
			return true;
		return false;
	};

	$scope.isPrefersMale = function() {
		if ($scope.userModel && $scope.userModel.orientation && $scope.userModel.orientation.charAt(1) == 'm')
			return true;
		return false;
	};

	$scope.updateOrientation = function() {
		//gender changed on usermodel, so need to change first char of orientation
		// var newOrientation = $scope.userModel.gender + $scope.userModel.orientation.charAt(1);
		$scope.userModel.orientation = $scope.userModel.gender + $scope.userModel.orientation.charAt(1)
	}

	$scope.setSmokes = function() {
		if ($scope.userModel)
			$scope.userModel.smokes = 'y';
	};

	$scope.setDontSmoke = function() {
		if ($scope.userModel)
			$scope.userModel.smokes = 'n';
	};

	$scope.isSmokes = function() {
		if ($scope.userModel && $scope.userModel.smokes == 'y')
			return true;
		return false;
	};

	$scope.isDontSmoke = function() {
		if ($scope.userModel && $scope.userModel.smokes == 'n')
			return true;
		return false;
	};

	$scope.editMedia = function(media) {
		//console.log("editMedia");
		AddMediaService.open($scope, media);
	}

	$scope.addMedia = function(mediaData) {
		AddMediaService.addMedia(mediaData, $scope);
	};
	
	
	$scope.expandTextArea = function(id) {
		var element = document.getElementById(id);
		if (element) {
			if (element.scrollHeight > _textAreaMaxHeight) {
				element.style.height =  _textAreaMaxHeight + "px";
			}
			else {
				element.style.height =  element.scrollHeight + "px";
			}
		}
	};


	function goHome() {
		$ionicHistory.nextViewOptions({
			disableBack: true
		});
		$timeout(function() { $state.go('home', { reload: true, cache: false }) }, 500);
	};


	function init() {
		if (!userModel) {
			goHome();
		}
		else {
			//$scope.userModel = userModel;
			UserService.syncUserModel($scope.userModel,
				function(userModel) {
					$scope.userModel = userModel;
				}
			);
		}

		ga.trackPageview('/profile');
	}

	init();
}]);


angular.module('hh.controllers')
.controller('ChatController', ['$cordovaEmailComposer', '$cordovaInAppBrowser', '$cordovaSocialSharing', '$http', '$ionicActionSheet', '$ionicHistory', '$ionicModal', '$ionicPlatform', '$ionicScrollDelegate', '$rootScope', '$scope', '$state', '$timeout', 'AddMediaService', 'AppConfig', 'MeetService', 'PushService', 'UserService', function($cordovaEmailComposer, $cordovaInAppBrowser, $cordovaSocialSharing, $http, $ionicActionSheet, $ionicHistory, $ionicModal, $ionicPlatform, $ionicScrollDelegate, $rootScope, $scope, $state, $timeout, AddMediaService, AppConfig, MeetService, PushService, UserService) {

	var userModel = UserService.getLoggedInUser();
	console.log("userModel", JSON.stringify(userModel));
	var timerCountDown;
	var timerGeoTime;
	var hourStart;
	var hourEnd;
	var currentDayOfWeek = (new Date()).getDay();
	var message = "";
	var chatroomNext = "";

	// scope variables

	$scope.appVersion = _appVersion;
	$scope.isAppUpdate = false;
	$scope.isGeoLocated = false;
	$scope.isShowCountDown = false;
	$scope.isPastTime = false;
	$scope.geoLocationString = "...";
	$scope.countDown = "...";
	$scope.userModel = userModel;
	$scope.textIsOver = "";
	$scope.textNextActivity = "";
	$scope.isPhoneCodeEdit = userModel.phoneNumber ? false : true;
	$scope.isViewLongBio = false;
	$scope.ages = [];
	for (var i = 18; i < 100; i++) {
		$scope.ages.push(i.toString());
	}
	$scope.isNewMessage = _isNewMessage;
	$scope.badgeNumber = _isNewMessage ? 1 : 0;
	$scope.phoneModel = {};
	
			
	// helpers
	
	function getCategoryByActivity() {
		if ($scope.userModel && $scope.userModel.activity)
			$scope.userModel.category = _categories[$scope.userModel.activity];
		return $scope.userModel.category;
	};


	function isOpen() {
		var hours = (new Date()).getHours();
		return !_openHours[$scope.userModel.activity] || (hours >= _openHours[$scope.userModel.activity].hourStart && hours < _openHours[$scope.userModel.activity].hourEnd);
	};


	function isOpenFor(activity) {
		var hours = (new Date()).getHours();
		return !_openHours[activity] || (hours >= _openHours[activity].hourStart && hours < _openHours[activity].hourEnd);
	};
	
	
	function isOpenBefore(activity) {
		var hours = (new Date()).getHours();
		return !_openHours[activity] || hours < _openHours[activity].hourStart;
	};
	

	function isOpenAfter(activity) {
		var hours = (new Date()).getHours();
		return !_openHours[activity] || hours >= _openHours[activity].hourEnd;
	};
	
	
	function setActivitiesAndCategory() {
		var dateNow = new Date();
		var dayOfWeek = dateNow.getDay();

		if (dayOfWeek == 0 || dayOfWeek == 6) {
			// brunch and drinks on weekend
			$scope.activities = [ { label: "\u00A0\u00A0Brunch", value: "brunch" }, { label: "\u00A0\u00A0Drinks", value: "drinks" } ];
			if (isOpenFor("brunch")) {
				$scope.userModel.activity = "brunch";
			}
			else if (isOpenFor("drinks")) {
				$scope.userModel.activity = "drinks";
			}
			else if (isOpenAfter("brunch")) {
				$scope.userModel.activity = "drinks";
			}
			else {
				$scope.userModel.activity = "brunch";
			}
		}
		else {
			// happy hour on weekdays
			$scope.activities = [ { label: "\u00A0\u00A0Happy Hour", value: "happy hour" } ];
			if (!$scope.userModel.activity || $scope.userModel.activity != "happy hour")
				$scope.userModel.activity = "happy hour";
		}
		
		$scope.userModel.category = getCategoryByActivity();
	};


	$scope.isOpenBefore = function(activity) {
		return isOpenBefore(activity);
	};


	$scope.isOpenAfter = function(activity) {
		return isOpenAfter(activity);
	};


	$scope.capitalize = function(string) {
		return string.replace(/\b\w/g, function(l){ return l.toUpperCase() })
	};
	
	
	$scope.getHourStartString = function(activity) {
		var hour = _openHours[activity].hourStart;
		if (hour == 0) {
			return "12am";
		}
		else if (!hour) {
			return "";
		}
		else if (hour == 12) {
			return hour + "pm";
		}
		else if (hour < 12) {
			return hour + "am";
		}
		else {
			return (hour - 12) + "pm";
		}
	};
	
	
	$scope.getHourEndString = function(activity) {
		var hour = _openHours[activity].hourEnd;
		if (hour == 0) {
			return "12am";
		}
		else if (!hour) {
			return "";
		}
		else if (hour == 12) {
			return hour + "pm";
		}
		else if (hour < 12) {
			return hour + "am";
		}
		else {
			return (hour - 12) + "pm";
		}
	};

	
	$scope.onChangeActivity = function() {
		setActivityTime();
		setCountDown();
	};
	
	
	function setActivityTime() {
		var isClose = !isOpen();
		// breakfast
		$scope.isNotBreakfastTime = ($scope.userModel.activity == "breakfast" && isClose);
		// lunch
		$scope.isNotLunchTime = ($scope.userModel.activity == "lunch" && isClose);
		// brunch
		$scope.isNotBrunchTime = ($scope.userModel.activity == "brunch" && isClose);
		// happy hour
		$scope.isNotHappyHour = ($scope.userModel.activity == "happy hour" && isClose);
		// drinks
		$scope.isNotDrinks = ($scope.userModel.activity == "drinks" && isClose);
	}
	

	$scope.setGeoLocation = function() {
		setGeoLocation();
	}


	function setGeoLocation() {
		if (AppConfig.isSimulator) {
			userModel.latitude = AppConfig.test.latitude;
			userModel.longitude = AppConfig.test.longitude;
			onGetGeoLocation({ coords: { latitude: userModel.latitude, longitude: userModel.longitude }});
		}
		else if (navigator.geolocation) {
			$scope.geoLocationString = "getting location...";
			navigator.geolocation.getCurrentPosition(onGetGeoLocation, onGetGeoLocationError);
		}
	}


	function onGetGeoLocation(geoLocation) {
		userModel.latitude = geoLocation.coords.latitude;
		userModel.longitude = geoLocation.coords.longitude;
		UserService.update({ latitude: userModel.latitude, longitude: userModel.longitude }, checkMeet);

		$scope.isGeoLocated = true;

		checkGeoTime();

		$http.get("https://maps.googleapis.com/maps/api/geocode/json?latlng=" + userModel.latitude + "," + userModel.longitude)
		.then(
			function(response) {
				//console.log('success', response);
				if (response.data && response.data.results[0] && response.data.results[0].formatted_address) {
					var address = response.data.results[0].formatted_address;
					var parts = address.split(",");
					var numParts = parts.length;
					if (numParts >= 3) {
						var country = parts[numParts - 1];
						var state = parts[numParts - 2];
						var city = parts[numParts - 3];
						if (city) {
							$scope.geoLocationString = city;
						}
					}
				}
			},
			function(err) {
				console.error('onGetGeoLocation error', err);
			}
		);
	};


	function onGetGeoLocationError(error) {
		$scope.isGeoLocated = false;
		console.log("onGetGeoLocationError", JSON.stringify(error));
		switch (error.code) {
			case error.PERMISSION_DENIED:
				if (window.cordova && ionic.Platform.isIOS()) {
					$scope.geoLocationString = "Turn on Location for JONES:<br/>Settings &rarr; Privacy &rarr; Location Services &rarr; JONES &rarr; While Using the App<br/>Tap here to try again.";
				}
				else if (ionic.Platform.isIOS() && navigator.userAgent.indexOf('Safari') != -1 && navigator.userAgent.indexOf('Chrome') == -1) {
					$scope.geoLocationString = "Turn on Location on Safari:<br/>Settings &rarr; Privacy &rarr; Location Services &rarr; Safari Websites &rarr; While Using the App<br/>Tap here to try again.";
				}
				else {
					$scope.geoLocationString = "Turn on Location for JONES<br/>Tap here to try again.";
				}
				break;
			default:
				$scope.$apply(function() { $scope.geoLocationString = "Turn on Location for JONES<br/>Tap here to try again."; });
		}
	};


	$scope.getFirstName = function(name) {
		if (!name)
			return "";
		var parts = name.split(" ");
		return parts[0];
	};


	$scope.getBio = function() {
		return $scope.profileUserModel.bio
			?	($scope.isViewLongBio
					? $scope.profileUserModel.bio
					: $scope.profileUserModel.bio.substring(0, _shortBioLength))
			:	"";
	};


	$scope.setBioLength = function() {
		$scope.isViewLongBio = !$scope.isViewLongBio;
		$scope.bio = $scope.getBio();
		$ionicScrollDelegate.resize();
		if (!$scope.isViewLongBio) {
			// hardcoding left and top works, but extracting from previousScrollPosition just goes to top or 0 offsets
			// $ionicScrollDelegate.scrollTo($scope.previousScrollPosition.left, $scope.previousScrollPosition.top, true);
			// $ionicScrollDelegate.scrollTo(15, 100, true);
			$ionicScrollDelegate.scrollTop(true);
		} else {
			$scope.previousScrollPosition = $ionicScrollDelegate.getScrollPosition();
		}
	};


	$scope.setEditing = function(attr) {
		$scope.editing = {};
		$scope.editing[attr] = true;
		cordova.plugins.Keyboard.show();
		console.log("show keyboard");
		var element = document.getElementById(attr);
		if (element) {
			element.focus();
			console.log("focus");
		}
	};


	$scope.getSwitchModel = function(attr) {
		return $scope.profileUserModel && $scope.profileUserModel.userId != $scope.userModel.userId ? $scope.profileUserModel[attr] : $scope.userModel[attr];
	};


	$scope.updateUser = function(attr) {
		if (attr == 'gender')
			$scope.updateOrientation();
		else if (attr == 'bio') {
			$scope.profileUserModel.bio = $scope.userModel.bio;
			$scope.bio = $scope.userModel.bio;
		}
		$scope.editing = {};
		UserService.update($scope.userModel);
	};


	$scope.updateOrientation = function() {
		//gender changed on usermodel, so need to change first char of orientation
		// var newOrientation = $scope.userModel.gender + $scope.userModel.orientation.charAt(1);
		$scope.userModel.orientation = $scope.userModel.gender + $scope.userModel.orientation.charAt(1)
	};


	$scope.showMoreOrLess = function() {
		return $scope.profileUserModel && $scope.profileUserModel.bio && $scope.profileUserModel.bio.length > _shortBioLength;
	};


	$scope.isAvailable = function(user) {
		if (!!user) {
			return !!user.isAvailable && !isPastOneHour(user.timestampAvailable);
		}
		return !!$scope.userModel.isAvailable && !isPastOneHour($scope.userModel.timestampAvailable);
	};
	
	
	function isPastOneHour(timestamp) {
		if (!timestamp)
			return true;
		var dateTime = new Date(timestamp);
		return ((new Date()).getTime() - dateTime.getTime()) / 3600 / 1000 > 1;
	};


	function setPushAvailable(param) {
		if (param == "never") {
			$scope.userModel.isPushAvailable = false;
		}
		else {
			$scope.userModel.isPushAvailable = true;
			$scope.userModel.pushAvailableWhen = param;
		}
		UserService.update({ isPushAvailable: $scope.userModel.isPushAvailable, pushAvailableWhen: $scope.userModel.pushAvailableWhen });
	};


	$scope.askPushAvailable = function() {
		var buttons = [
				{ text: "any time" },
				{ text: "evening" },
				{ text: "weekend" },
				{ text: "when I'm live" },
				{ text: "never" }
		];
		
		$ionicActionSheet.show({
			titleText: "Notify me who is live and ready to meet",
			buttons: buttons,
			buttonClicked: function(index) {
				switch(index) {
					case 0:
						setPushAvailable("anytime");
						break;
					case 1:
						setPushAvailable("evening");
						break;
					case 2:
						setPushAvailable("weekend");
						break;
					case 3:
						setPushAvailable("live");
						break;
					case 4:
						setPushAvailable("never");
						break;
				}
				ga.trackEvent('setPushAvailable', buttons[index].text, 'userId', userModel.userId);
				return true;
			}
		});
		
		closeDrawer();
	};
	
	
	$scope.expandTextArea = function(id) {
		var element = document.getElementById(id);
		if (element) {
			if (element.scrollHeight > _textAreaMaxHeight) {
				element.style.height =  _textAreaMaxHeight + "px";
			}
			else {
				element.style.height =  element.scrollHeight + "px";
			}
		}
	};


	$scope.getMoreOrLess = function() {
		return $scope.isViewLongBio ?
			" (see less)" :
			"... (see more)";
	};


	$scope.getUrlPhoto = function(urlPhoto, size) {
		if (!size)
			size = 180;
		if (urlPhoto)
			return  urlPhoto.replace("http:", "https:") + (urlPhoto.indexOf("graph.facebook") != -1 ? "?type=square&width=" + size + "&height=" + size : "");
		else
			return "";
	};


	$scope.getUrlPhotoSelf = function(size) {
		if (userModel && userModel.urlPhoto)
			return $scope.getUrlPhoto(userModel.urlPhoto, size);
		else
			return "";
	};


	$scope.getDateTimeString = function(timestamp) {
		if (!timestamp) {
			return "";
		}
		var dateTimeString = new Date(timestamp).toString();
		return (dateTimeString.substring(0, dateTimeString.indexOf(":") + 3));
	}
	

	$scope.openProfile = function(user) {
		closeDrawer();

		if ($scope.profileUserModal && $scope.profileUserModel && user.userId == $scope.profileUserModel.userId) {
			return;
		}

		$scope.closeProfile();
		
		$ionicModal.fromTemplateUrl('views/profile.html', {
			scope: $scope,
			animation: 'slide-in-up'
		}).then(function(modal) {
			$scope.profileUserModal = modal;
			var params = { userId: user.userId };
			UserService.getUser(params).then(
				function(response) {
					$scope.profileUserModel = response.data;
					$scope.editing = {}
					$scope.isViewLongBio = false;
					$scope.bio = $scope.getBio();
					$scope.profileUserModal.show();

					if (ionic.Platform.isAndroid()) {
						$ionicPlatform.onHardwareBackButton(function() {
							$scope.closeProfile();
						});
					}
					
					//console.log($scope.profileUserModel);
				}
			);
		});
		
		// get mutual friends if not self
		if (user.userId != userModel.userId) {
			UserService.getMutualFriends({ userId: user.userId }).then(
				function(response) {
					$scope.mutualFriends = response.data && response.data.length > 0
						? response.data
						: null;
				}
			);
		}

		// want userId of profile viewed?
		ga.trackEvent('profileView', 'openProfile', 'userId', userModel.userId);
	};


	$scope.closeProfile = function() {
		if ($scope.profileUserModal) {
			$scope.isViewLongBio = false;
			$scope.bio = null;
			$scope.profileUserModal.remove();
			$scope.profileUserModal = null;
		}
	};


	$scope.editProfile = function() {
		$scope.closeProfile();
		$timeout(function() { $state.go('profile'); }, 100 );
	};


	$scope.editMedia = function(media) {
		//console.log("editMedia");
		AddMediaService.open($scope, media);
	}


	$scope.addMedia = function(mediaData) {
		AddMediaService.addMedia(mediaData, $scope);
	};
	
	
	$scope.showNotTime = function() {
		if ($scope.notTimeModal)
			$scope.notTimeModal.close();

		$ionicModal.fromTemplateUrl('views/not-time.html', {
			scope: $scope,
			animation: 'slide-in-right'
		}).then(function(modal) {
			$scope.notTimeModal = modal;
			$scope.notTimeModal.show();
			if (ionic.Platform.isAndroid()) {
				$ionicPlatform.onHardwareBackButton(function() {
					$scope.closeNotTime();
				});
			}
		});
	};
	
	
	$scope.closeNotTime = function() {
		if ($scope.notTimeModal) {
			$scope.notTimeModal.remove();
			$scope.notTimeModal = null;
		}
	};
	
	
	// messages
	
	function setMessageBadgeNumber(badgeNumber) {
		$scope.badgeNumber = badgeNumber;
		_isNewMessage = badgeNumber > 0;
		$scope.isNewMessage = _isNewMessage;
	}


	function checkNewMessage() {
		UserService.getMessages({ indexLimit: 1 }).then(
			function(response) {
				if (response.data && response.data.length > 0) {
					var lastMessage = response.data[0];
					if (!$scope.userModel.lastMessageId || lastMessage.messageId != $scope.userModel.lastMessageId) {
						setMessageBadgeNumber(1);
						UserService.update({ lastMessageId: lastMessage.messageId });
					}
				}
			}
		);
	}
	

	$scope.showMessages = function() {
		if ($scope.messagesModal)
			$scope.closeMessages();

		setMessageBadgeNumber(0);

		$scope.getMessages();

		$ionicModal.fromTemplateUrl('views/messages.html', {
			scope: $scope,
			animation: 'slide-in-up'
		}).then(function(modal) {
			$scope.messagesModal = modal;
			$scope.messagesModal.show();
			if (ionic.Platform.isAndroid()) {
				$ionicPlatform.onHardwareBackButton(function() {
					$scope.closeMessages();
				});
			}
		});
	}
	
	
	$scope.closeMessages = function() {
		if ($scope.messagesModal) {
			$scope.messagesModal.remove();
			$scope.messagesModal = null;
			$scope.messages = [];
		}
	};
	
	
	$scope.getMessages = function(paramsIn) {
		var params = { indexLimit: _indexLimitMessages };

		if (paramsIn && paramsIn.isMore) {
			params.indexStart = $scope.messages.length;
		}
		else {
			$scope.isShowMoreMessages = false;
			$scope.messages = [];
		}
		
		UserService.getMessages(params).then(
			function(response) {
				var numMessages = (response.data && response.data.length > 0) ? response.data.length : 0;
				if (!$scope.messages)
					$scope.messages = [];
				if (numMessages > 0) {
					for (i = 0; i < numMessages; i++) {
						$scope.messages.push(response.data[i]);
					}
				}
				$scope.isShowMoreMessages = !(numMessages == 0 || numMessages < _indexLimitMessages);
				$scope.$broadcast('scroll.refreshComplete');
				$scope.$broadcast('scroll.infiniteScrollComplete');
			},
			function(error) {
				$scope.isShowMoreMessages = false;
				$scope.$broadcast('scroll.refreshComplete');
				$scope.$broadcast('scroll.infiniteScrollComplete');
			}
		);
	};


	// faq

	$scope.showFaq = function() {
		$cordovaInAppBrowser.open(AppConfig.urlFAQ + '?_=' + (new Date()).getTime(), 'jones', { allowInlineMediaPlayback: "yes" });
		ga.trackEvent('faq', 'main', 'userId', userModel.userId);
		closeDrawer();
		return false;
	};


	// feedback

	$scope.giveFeedback = function() {
		$cordovaEmailComposer.isAvailable().then(
			function() {
				var email = {
					to: 'support@joinjones.com',
					subject: 'Feedback for JONES v' + _appVersion + ' ' + ($scope.userModel && $scope.userModel.userId ? $scope.userModel.userId : ''),
					body: '',
					isHtml: false
				};

				$cordovaEmailComposer.open(email);
			},
			function () {
				// default email not set up or not available
				window.open('mailto:support@joinjones.com?subject=Feedback for JONES v' + _appVersion + ' ' + ($scope.userModel && $scope.userModel.userId ? $scope.userModel.userId : ''), '_system');
			}
		);
		
		ga.trackEvent('feedback', 'main', 'userId', userModel.userId);
		closeDrawer();
		return false;
	};


	// tell friends

	$scope.tellFriends = function() {
		console.log("tellfriends");
		var appLink = ionic.Platform.isAndroid() ? AppConfig.urlApp.android : AppConfig.urlApp.ios;
		$cordovaSocialSharing.share("Join me for drinks & socializing on JONES app", appLink, null, appLink);
		ga.trackEvent('tellFriends', 'main', 'userId', userModel.userId);
		closeDrawer();
		return false;
	};


	// sign out

	$scope.signOut = function() {
		confirmLogout();
		closeDrawer();
	};


	function confirmLogout() {
		navigator.notification.confirm(
			"",
			function(index) {
				switch(index) {
					case 2:
						var userId = userModel.userId;
						logout();
						ga.trackEvent('logout', 'main', 'userId', userId);
				}
			},
			"Sign Out?",
			["No", "Yes"]
		);
	};


	function logout() {
		_isInitPush = false;
		UserService.logout();
		goHome();
	}
	
	
	// close drawer
	
	function closeDrawer() {
		$timeout(function() { $scope.closeDrawer(); }, 300);
	};
	
	
	// update app
	
	$scope.updateApp = function() {
		if (ionic.Platform.isIOS()) {
			window.open(_urlUpdateAppiOS);
		}
		else if (ionic.Platform.isAndroid()) {
			window.plugins.webintent.startActivity({
					action: window.plugins.webintent.ACTION_VIEW,
					url: _urlUpdateAppGooglePlay
				},
				function() { console.log('success'); },
				function(errorMsg) { console.log('error: ' + errorMsg); }
			);
		}
	};


	function showUpdateApp() {
		if (!navigator.notification)
			return;

		navigator.notification.confirm(
			"There is a new version of the app.",
			function(index) {
				switch(index) {
					case 1:
						break;
					case 2:
						$scope.updateApp();
						break;
				}
			},
			"Update App?",
			["Later", "Now"]
		);
	};
	

	// confirm phone number

	$scope.openConfirmPhoneNumberModal = function() {
		_isShownConfirmPhoneNumber = true;
		$scope.isPhoneCodeSent = false;
		$scope.isPhoneCodeError = false;
		$scope.isPhoneCodeEdit = $scope.userModel.phoneNumber ? false : true;
		$scope.isPhoneNumberConfirmed = false;
		$scope.closeConfirmPhoneNumberModal();
		$scope.phoneModel.isPushText = $scope.userModel.hasOwnProperty("isPushText") ? $scope.userModel.isPushText : true;
		$ionicModal.fromTemplateUrl('views/confirm-phone.html', {
			scope: $scope,
			animation: 'slide-in-up'
		}).then(function(modal) {
			$scope.confirmPhoneNumberModal = modal;
			$scope.confirmPhoneNumberModal.show();
		});
	};


	$scope.closeConfirmPhoneNumberModal = function() {
		if ($scope.confirmPhoneNumberModal)
			$scope.confirmPhoneNumberModal.remove();
	};


	$scope.updateUserIsPushText = function(isPushText) {
		UserService.update({ isPushText: isPushText });
		$scope.userModel.isPushText = isPushText;
	};


	$scope.phoneUser = function() {
		delete $scope.userModel.phoneCode;
		delete $scope.phoneModel.phoneCode;
		$scope.isPhoneCodeSent = false;
		$scope.isPhoneCodeError = false;
		$scope.isPhoneCodeEdit = false;
		$scope.isPhoneNumberConfirmed = false;
		if (!$scope.userModel.phoneUnconfirmed || String($scope.userModel.phoneUnconfirmed).length < 10)
			return;

		//console.log('phoneUser: ' + $scope.userModel.phoneUnconfirmed + ' ' + String($scope.userModel.phoneUnconfirmed).length);

		var params = { phoneUnconfirmed: String($scope.userModel.phoneUnconfirmed) };
		UserService.phoneUser(params,
			function(response) {
				if (response.r && response.r == 's')
					$scope.isPhoneCodeSent = true;
			}
		);
	};


	$scope.confirmUser = function() {
		if (!$scope.phoneModel.phoneCode || String($scope.phoneModel.phoneCode).length < 5)
			return;

		var params = { phoneCode: String($scope.phoneModel.phoneCode) };
		if ($scope.phoneModel.isPushText) {
			params.isPushText = true;
		}
		UserService.confirmUser(params,
			function(response) {
				if (response.r && response.r == 's') {
					$scope.isPhoneNumberConfirmed = true;
					$timeout(
						function() {
							$scope.closeConfirmPhoneNumberModal();
							if (chatroomNext == "myfriends") {
								chatroomNext = "";
								goChatroom("myfriends");
							}
						},
						2000
					);					
				}
				else {
					$scope.isPhoneCodeError = true;
				}
			}
		);
	};
	
	
	//  chatroom

	function goHome() {
		$ionicHistory.nextViewOptions({
			disableBack: true
		});
		$timeout(function() { $state.go('home', { reload: true, cache: false }) }, 500);
	};


	$scope.goChatroom = function(chatroom) {

		if (chatroom == 'myfriends') {
			UserService.confirmedUser(
				function(response) {
					if (response && response.r == 's') {
						if (!isOpen()) {
							$scope.showNotTime();
							return;
						}
						goChatroom(chatroom);
					}
					else {
						chatroomNext = "myfriends";
						$scope.openConfirmPhoneNumberModal();
					}
				}
			);
			return;
		}

		if (!isOpen()) {
			$scope.showNotTime();
			return;
		}

		goChatroom(chatroom);
	};


	function goChatroom(chatroom) {

		getCategoryByActivity();
		UserService.update({ chatroom: chatroom, category: $scope.userModel.category, activity: $scope.userModel.activity });
		ga.trackEvent('chatroom', chatroom);
		ga.trackEvent('category', $scope.userModel.category);
		ga.trackEvent('activity', $scope.userModel.activity);
		$timeout(function() {
			$state.go("chat/chatroom", { chatroom: chatroom, category: $scope.userModel.category, activity: $scope.userModel.activity }, { reload: true });
		}, 100);
	};


	function checkMeet() {
		//console.log("checkMeet _urlScheme", _urlScheme);
		$scope.onChangeActivity();

		checkGeoTime();

		//checkNewMessage();
			
		if (_urlScheme) {
			var params = getParamsObjFromUrl(_urlScheme);
			//console.log("params", JSON.stringify(params));
			if (params) {
				if (params.meetId) {
					// see someone I'm meeting at meetId
					getMeet(params);
				}
				else if (params.likeUserId) {
					// see someone who likes me
					processLike(params);
				}
				return;
			}
		}

 		getMeet({ userId: $scope.userModel.userId });
	};


	function getMeet(params) {
		// check if already meeting someone
		MeetService.getMeet(params)
		.then(
			function(response) {
				console.log("getMeet", JSON.stringify(response));
				if (response.r && response.r == "s" && response.data) {
					processMeet(response.data);
				}
				else {
					checkLike();
				}
			}
		);
	};


	function processMeet(meet) {
		if (meet.chatroom) {
			goChatroom(meet.chatroom);
		}
		else {
			checkLike();
		}
	};


	function checkLike() {
		MeetService.getWhoLikesMe({ isViewed: { $exists: false }, indexLimit: 1 })
		.then(
			function(response) {
				//console.log("getWhoLikesMe", JSON.stringify(response));
				if (response.r && response.r == "s" && response.data && response.data.length > 0) {
					var data = response.data[0];
					processLike({ likeUserId: data.userId });
				}
			}
		);
	};


	function processLike(params) {
		var userParams = { userId: params.likeUserID };
		UserService.getUser(userParams)
		.then(
			function(data) {
				//console.log("processLike", JSON.stringify(data));
				if (data.r && data.r == "s") {
					var profileUserModel = data.data;
					if (profileUserModel.chatroom) {
						goChatroom(profileUserModel.chatroom);
					}
				}
			}
		);
	};


	function checkGeoTime() {
		UserService.getUserStatus()
		.then(
			function(response) {
				//console.log(response);
				if (response.r == "s" && response.data) {
					var data = response.data;

					$scope.isRestrictGeo = AppConfig.isRestrictGeo ? (data.isRestrictGeo ? true : false) : false;
					
					initCountDown();
					
					if (timerGeoTime)
						clearInterval(timerGeoTime);
					timerGeoTime = setInterval(checkGeoTime, 600000);
					$scope.isAppUpdate = !!data.isAppUpdate;
					if ($scope.isAppUpdate && !_isShownUpdateApp) {
						_isShownUpdateApp = true;
						showUpdateApp();
					}
				}
			}
		);
	};


	function checkTime() {
		var dateNow = new Date();
		return (dateNow.getHours() >= hourStart && dateNow.getHours() < hourEnd);
	};


	function setCountDown() {
		var dateNow = new Date();

		if (dateNow.getDay() != currentDayOfWeek) {
			setActivitiesAndCategory();
		}

		if (_openHours[$scope.userModel.activity]) {
			hourStart = _openHours[$scope.userModel.activity].hourStart;
			hourEnd = _openHours[$scope.userModel.activity].hourEnd;
		}

		if (dateNow.getHours() < hourStart) {
			// before start
			if ($scope.textIsOver != "")
				$scope.textIsOver = "";
			if ($scope.textNextActivity != "")
				$scope.textNextActivity = "";
			if (!$scope.isShowCountDown)
				$scope.isShowCountDown = true;
			$scope.nextActivity = $scope.userModel.activity;
		}
		else if (dateNow.getHours() >= hourStart && dateNow.getHours() < hourEnd) {
			// started but not ended yet
			if ($scope.isShowCountDown)
				$scope.isShowCountDown = false;
			if ($scope.textNextActivity != "")
				$scope.textNextActivity = "";
			if (timerCountDown)
				clearInterval(timerCountDown);
			timerCountDown = null;
			setActivityTime();
			$scope.nextActivity = $scope.userModel.activity;
		}
		else {
			// ended
			if (dateNow.getDay() == 0 || dateNow.getDay() == 6) {
				if (isOpenAfter("brunch") && isOpenBefore("drinks")) {
					$scope.nextActivity = "drinks";
					if ($scope.textNextActivity != "Next Drinks ")
						$scope.textNextActivity = "Next Drinks ";
				}
				else {
					$scope.nextActivity = "happy hour";
					if ($scope.textNextActivity != "Next Happy Hour ")
						$scope.textNextActivity = "Next Happy Hour ";
				}
			}
			else if (dateNow.getDay == 5) {
				$scope.nextActivity = "brunch";
				if ($scope.textNextActivity != "Next Brunch ")
					$scope.textNextActivity = "Next Brunch ";
			}
			else {
				$scope.nextActivity = "happy hour";
				if ($scope.textNextActivity != "Next Happy Hour ")
					$scope.textNextActivity = "Next Happy Hour ";
			}
			
			if ($scope.activity != $scope.nextActivity && _openHours[$scope.nextActivity]) {
				hourStart = _openHours[$scope.nextActivity].hourStart;
				hourEnd = _openHours[$scope.nextActivity].hourEnd;
			}
			
			if ($scope.textIsOver != " is over.")
				$scope.textIsOver = " is over.";
			if (!$scope.isShowCountDown)
				$scope.isShowCountDown = true;
		}

		var hours = (24 + hourStart - dateNow.getHours() - (dateNow.getMinutes() == 0 && dateNow.getSeconds() == 0 ? 0 : 1)) % 24;
		var minutes = (60 - dateNow.getMinutes() - (dateNow.getSeconds() == 0 ? 0 : 1)) % 60;
		var seconds = (60 - dateNow.getSeconds()) % 60;

		var countDown =
			(hours > 0 ? '<span class="white">' + hours + '</span><span class="gray">h</span>' : "")
			+ (minutes > 0 ? ' <span class="white">' + minutes + '</span><span class="gray">m</span>' : "")
			+ (seconds > 0 ? ' <span class="white">' + seconds + '</span><span class="gray">s</span>' : "");
		
		$scope.$evalAsync(function() { $scope.countDown = countDown; });
	};


	function initCountDown() {
		setCountDown();
		if (!timerCountDown)
			timerCountDown = setInterval(setCountDown, 1000);
	};
	

	function showMessage(params) {
		if (params.text) {
			navigator.notification.alert(params.text);
		}
	};
	
	
	$scope.openMessage = function(message) {
		if (!message) {
			return;
		}
		if (message.fromUserId != $scope.userModel.userId) {
			$scope.openProfile({ userId: message.fromUserId }, false, true);
		}
		else if (message.toUserId != $scope.userModel.userId) {
			$scope.openProfile({ userId: message.toUserId }, true, false);
		}
	}


	// events

	var offCheckMeet = $rootScope.$on('checkMeet', function(event, params) {
		checkMeet();
	});


	var offInvitePush = $rootScope.$on('invitePush', function(event, params) {
		checkMeet();
	});


	var offMessagePush = $rootScope.$on('messagePush', function(event, params) {
		showMessage(params);
	});


	$scope.$on('$destroy', function() {
		offCheckMeet();
		offInvitePush();
		offMessagePush();

		if (timerCountDown) {
			clearInterval(timerCountDown);
			timerCountDown = null;
		}

		if (timerGeoTime) {
			clearInterval(timerGeoTime);
			timerGeoTime = null;
		}

		$scope.closeProfile();
		$scope.closeConfirmPhoneNumberModal();
	});


	// init

	function init() {
		if (!userModel) {
			goHome();
			return;
		}

		UserService.syncUserModel($scope.userModel);

		setActivitiesAndCategory();

		$scope.onChangeActivity();
		
		setGeoLocation();
		
		if (!_isInitPush) {
			$ionicPlatform.ready(function() {
				_isInitPush = true;
				PushService.init();
			});
		}
				
		ga.trackPageview('/main');
	}

	init();
}]);


angular.module('hh.controllers')
.controller('ChatroomController', ['$cordovaEmailComposer', '$cordovaInAppBrowser', '$cordovaSocialSharing', '$cordovaContacts', '$ionicActionSheet', '$ionicHistory', '$ionicModal', '$ionicPlatform', '$ionicPopup', '$ionicScrollDelegate', '$rootScope', '$scope', '$state', '$stateParams', '$timeout', 'AddMediaService', 'AppConfig', 'FacebookService', 'MeetService', 'UserService', function($cordovaEmailComposer, $cordovaInAppBrowser, $cordovaSocialSharing, $cordovaContacts, $ionicActionSheet, $ionicHistory, $ionicModal, $ionicPlatform, $ionicPopup, $ionicScrollDelegate, $rootScope, $scope, $state, $stateParams, $timeout, AddMediaService, AppConfig, FacebookService, MeetService, UserService) {

	var userModel = UserService.getLoggedInUser();
	var currentChatroomUsers = [];
	var meet;
	var alertPopup;
	var timerGeo;
	var timerMeet;
	var invites = [];
	var eventIds = {};
	var indexStartUsers = 0;
	var lastWeight = 0;
	var lastTimestamp = null;
	
	// scope variables

	$scope.contacts = [];
	$scope.isCancelingMeet = false;
	$scope.isRainCheckingMeet = false;
	$scope.isSelfConfirmed = false;
	$scope.isMeetConfirmed = false;
	$scope.isSelfCheckedIn = false;
	$scope.isMeetCheckedIn = false;
	$scope.userModel = userModel;
	$scope.profileUserModel = {};
	$scope.meetUserModel = {};
	$scope.phoneModel = {};
	$scope.users = [];
	$scope.messages = [];
	$scope.placeName = "";
	$scope.placeAddress = "";
	$scope.isHappyHour = false;
	$scope.isBrunch = false;
	$scope.isViewLongBio = false;
	$scope.isShowMoreUsers = false;
	$scope.isShowMoreMessages = false;
	$scope.isNoOneNearby = false;
	$scope.isNewMessage = _isNewMessage;
	$scope.badgeNumber = _isNewMessage ? 1 : 0;


	// profile

	$scope.ages = [];
	for (var i = 18; i < 100; i++) {
		$scope.ages.push(i.toString());
	}


	$scope.setEditing = function(attr) {
		$scope.editing = {};
		$scope.editing[attr] = true;
		cordova.plugins.Keyboard.show();
		console.log("show keyboard");
		var element = document.getElementById(attr);
		if (element) {
			element.focus();
			console.log("focus");
		}
	}
								   

	$scope.getSwitchModel = function(attr) {
		return $scope.profileUserModel && $scope.profileUserModel.userId != $scope.userModel.userId ? $scope.profileUserModel[attr] : $scope.userModel[attr];
	}
	
								   
	$scope.updateUser = function(attr) {
		if (attr == 'gender')
			$scope.updateOrientation();
		else if (attr == 'bio') {
			$scope.profileUserModel.bio = $scope.userModel.bio;
			$scope.bio = $scope.userModel.bio;
		}
		$scope.editing = {};
		UserService.update($scope.userModel)
	}
	
								   
	$scope.updateOrientation = function() {
		//gender changed on usermodel, so need to change first char of orientation
		// var newOrientation = $scope.userModel.gender + $scope.userModel.orientation.charAt(1);
		$scope.userModel.orientation = $scope.userModel.gender + $scope.userModel.orientation.charAt(1)
	}
	
	
	$scope.expandTextArea = function(id) {
		var element = document.getElementById(id);
		if (element) {
			if (element.scrollHeight > _textAreaMaxHeight) {
				element.style.height =  _textAreaMaxHeight + "px";
			}
			else {
				element.style.height =  element.scrollHeight + "px";
			}
		}
	};


	// available

	$scope.isAvailable = function(user) {
		if (!!user) {
			return !!user.isAvailable && !isPastOneHour(user.timestampAvailable);
		}
		return !!$scope.userModel.isAvailable && !isPastOneHour($scope.userModel.timestampAvailable);
	};


	function setAvailable(isAvailable) {
		if (isAvailable) {
			$scope.userModel.isAvailable = true;
			$scope.userModel.timestampAvailable = (new Date()).toISOString();
		}
		else {
			$scope.userModel.isAvailable = false;
		}

		var data = { isAvailable: $scope.userModel.isAvailable };
		if ($scope.userModel.isAvailable) {
			data.timestampAvailable = $scope.userModel.timestampAvailable;
		}
		UserService.update(data);
	};


	function checkAvailable() {
		if (!$scope.userModel.hasOwnProperty("isAvailable")) {
			$scope.askAvailable();
		}
		else {
			setAvailable($scope.userModel.isAvailable);
			checkPushAvailable();
		}
	};


	$scope.askAvailable = function() {
		if (!navigator.notification)
			return;

		navigator.notification.confirm(
			"Press YES to activate your purple circle and let others know you’re ready.",
			function(index) {
				switch(index) {
					case 1:
						setAvailable(false);
						break;
					case 2:
						setAvailable(true);
						break;
				}
				setTimeout(checkPushAvailable, 1000);
			},
			"JONESING to meet in the next hour?",
			["Not Yet", "Yes"]
		);
	};


	$scope.toggleAvailable = function(availableStatus) {
		setAvailable(availableStatus)
	};


	// push available

	$scope.isPushAvailable = function() {
		return !!$scope.userModel.isPushAvailable;
	};


	function setPushAvailable(param) {
		if (param == "never") {
			$scope.userModel.isPushAvailable = false;
		}
		else {
			$scope.userModel.isPushAvailable = true;
			$scope.userModel.pushAvailableWhen = param;
		}
		UserService.update({ isPushAvailable: $scope.userModel.isPushAvailable, pushAvailableWhen: $scope.userModel.pushAvailableWhen });
	};


	function checkPushAvailable() {
		if (!$scope.userModel.hasOwnProperty("isPushAvailable")) {
			$scope.askPushAvailable();
		}
	};


	$scope.askPushAvailable = function() {
		var buttons = [
				{ text: "any time" },
				{ text: "evening" },
				{ text: "weekend" },
				{ text: "when I'm live" },
				{ text: "never" }
		];
		
		$ionicActionSheet.show({
			titleText: "Notify me who is live and ready to meet",
			buttons: buttons,
			buttonClicked: function(index) {
				switch(index) {
					case 0:
						setPushAvailable("anytime");
						break;
					case 1:
						setPushAvailable("evening");
						break;
					case 2:
						setPushAvailable("weekend");
						break;
					case 3:
						setPushAvailable("live");
						break;
					case 4:
						setPushAvailable("never");
						break;
				}
				ga.trackEvent('setPushAvailable', buttons[index].text, 'userId', userModel.userId);
				return true;
			}
		});
		
		closeDrawer();
	};


	// users

	function isMatchOrientation(data) {
		if (userModel.chatroom == 'romance')
			return (data && data.orientation == getMatchOrientation());
		return true;
	};


	function addUser(data) {
		if (data.userId) {
			if (currentChatroomUsers[data.userId]) {
				currentChatroomUsers[data.userId] = data;
				replaceUser(data);
				return;
			}
			else if (data.userId == userModel.userId || !isMatchOrientation(data)) {
				return;
			}
			currentChatroomUsers[data.userId] = data;
		}
		else if (data.phoneNumber) {
			if (currentChatroomUsers[data.phoneNumber]) {
				currentChatroomUsers[data.phoneNumber] = data;
				replaceUser(data);
				return;
			}
			currentChatroomUsers[data.phoneNumber] = data;
		}
		$scope.users.unshift(data);
	};


	function appendUser(data) {
		if (data.userId) {
			if (data.userId == userModel.userId || currentChatroomUsers[data.userId] ||
				!isMatchOrientation(data))
				return;
			currentChatroomUsers[data.userId] = data;
		}
		if (data.phoneNumber) {
			if (currentChatroomUsers[data.phoneNumber])
				return;
			currentChatroomUsers[data.phoneNumber] = data;
		}
		$scope.users.push(data);
	};


	function addUsers(data) {
		var length = data.length;
		for (var i = 0; i < length; i++) {
			//console.log("addUsers: ", JSON.stringify(data[i]));
			addUser(data[i]);
		}
	};


	function appendUsers(data) {
		var length = data.length;
		for (var i = 0; i < length; i++) {
			//console.log("appendUsers: ", JSON.stringify(data[i]));
			appendUser(data[i]);
		}
	};


	function clearUsers() {
		$scope.users = [];
		currentChatroomUsers = [];
	};


	function removeUser(data) {
		var length = $scope.users.length;
		for (var i = 0; i < length; i++) {
			if (($scope.users[i].userId && $scope.users[i].userId == data.userId) || ($scope.users[i].phoneNumber && $scope.users[i].phoneNumber == data.phoneNumber)) {
				$scope.users.splice(i, 1);
				if (data.userId)
					delete currentChatroomUsers[data.userId];
				if (data.phoneNumber)
					delete currentChatroomUsers[data.phoneNumber];
				break;
			}
		}
	};


	function replaceUser(data) {
		console.log("replaceUser", JSON.stringify(data));
		var length = $scope.users.length;
		for (var i = 0; i < length; i++) {
			if (($scope.users[i].userId && $scope.users[i].userId == data.userId) || ($scope.users[i].phoneNumber && $scope.users[i].phoneNumber == data.phoneNumber)) {
				$scope.users[i] = angular.extend($scope.users[i], data);
				break;
			}
		}
	};


	function likeUser(data) {
		var length = $scope.users.length;
		for (var i = 0; i < length; i++) {
			if ((data.userId && $scope.users[i].userId && $scope.users[i].userId == data.userId) || (data.phoneNumber && $scope.users[i].phoneNumber && $scope.users[i].phoneNumber == data.phoneNumber)) {
				$scope.users[i].isInvited = true;
				break;
			}
		}
	};


	function resetIsInvited(data) {
		for (var i = 0; i < $scope.users.length; i++) {
			if (($scope.users[i].userId && $scope.users[i].userId == data.userId) || ($scope.users[i].phoneNumber && $scope.users[i].phoneNumber == data.phoneNumber)) {
				$scope.users[i].isInvited = false;
				break;
			}
		}
	};


	// contacts

	function addContact(contact) {
		$scope.contacts.unshift(contact);
	};


	function clearContacts() {
		$scope.contacts = [];
	};


	function removeContact(contact) {
		var length = $scope.contacts.length;
		for (var i = 0; i < length; i++) {
			if ($scope.contacts[i].id == contact.id) {
				$scope.contacts.splice(i, 1);
				break;
			}
		}
	};


	// first name

	$scope.getFirstName = function(name) {
		if (!name)
			return "";
		var parts = name.split(" ");
		return parts[0];
	};


	$scope.getBio = function() {
		return $scope.profileUserModel.bio
			?	($scope.isViewLongBio
					? $scope.profileUserModel.bio
					: $scope.profileUserModel.bio.substring(0, _shortBioLength))
			:	"";
	};


	$scope.setBioLength = function() {
		$scope.isViewLongBio = !$scope.isViewLongBio;
		$scope.bio = $scope.getBio();
		$ionicScrollDelegate.resize();
		if (!$scope.isViewLongBio) {
			// hardcoding left and top works, but extracting from previousScrollPosition just goes to top or 0 offsets
			// $ionicScrollDelegate.scrollTo($scope.previousScrollPosition.left, $scope.previousScrollPosition.top, true);
			// $ionicScrollDelegate.scrollTo(15, 100, true);
			$ionicScrollDelegate.scrollTop();
		} else {
			$scope.previousScrollPosition = $ionicScrollDelegate.getScrollPosition();
		}
	};


	$scope.showMoreOrLess = function() {
		return $scope.profileUserModel && $scope.profileUserModel.bio && $scope.profileUserModel.bio.length > _shortBioLength;
	};


	$scope.getMoreOrLess = function() {
		return $scope.isViewLongBio ?
			" (see less)" :
			"... (see more)";
	};


	$scope.getUrlPhoto = function(urlPhoto, size) {
		if (!size)
			size = 180;
		return  urlPhoto.replace("http:", "https:") + (urlPhoto.indexOf("graph.facebook") != -1 ? "?type=square&width=" + size + "&height=" + size : "");
	};


	$scope.getUrlPhotoSelf = function(size) {
		return (userModel && userModel.urlPhoto)
			? $scope.getUrlPhoto(userModel.urlPhoto, size)
			: "";
	};


	function getTimeOnly(dateTimeIn) {
		var dateTime = new Date(dateTimeIn.getTime());
		return (dateTime.getHours() > 12 ? dateTime.getHours() - 12 : dateTime.getHours())
			+ ":" + (dateTime.getMinutes() < 10 ? "0" + dateTime.getMinutes() : dateTime.getMinutes());
	};


	function getAMPM(dateTime) {
		return (dateTime.getHours() >= 12 ? "pm" : "am");
	};


	function isPastOneHour(timestamp) {
		if (!timestamp)
			return true;
		var dateTime = new Date(timestamp);
		return ((new Date()).getTime() - dateTime.getTime()) / 3600 / 1000 > 1;
	};


	$scope.getMeetTimeOnly = function() {
		if (!$scope.meetUserModel || !$scope.meetUserModel.timestamp) {
			return "";
		}
		var meetDateTime = new Date($scope.meetUserModel.timestamp);
		return getTimeOnly(meetDateTime);
	};


	$scope.getMeetAMPM = function() {
		if (!$scope.meetUserModel || !$scope.meetUserModel.timestamp) {
			return "";
		}
		var meetDateTime = new Date($scope.meetUserModel.timestamp);
		return getAMPM(meetDateTime);
	};

	
	$scope.getDateTimeString = function(timestamp) {
		if (!timestamp) {
			return "";
		}
		var dateTimeString = new Date(timestamp).toString();
		return (dateTimeString.substring(0, dateTimeString.indexOf(":") + 3));
	}


	function onNewUser(data) {
		//console.log("event:new:user", JSON.stringify(data));

		if (userModel.chatroom == "myfriends")
			return;

		$scope.$apply(function() {
			addUser(data);
			//console.log("new user users", JSON.stringify($scope.users));
		});
	};


	function connectUser() {
		//console.log("connectUser: emit", JSON.stringify(userModel));

		_socket.emit('event:connect:user', JSON.stringify(userModel), onConnectUser);
	};


	// get currently connected users
	function onConnectUser(data) {
		//console.log("onConnect:", data);

		$scope.getUsers();
		checkMeet();
	};


	function getMatchOrientation() {
		if (userModel.orientation && userModel.orientation.length == 2) {
			return userModel.orientation.charAt(1) + userModel.orientation.charAt(0);
		}
		else if (userModel.gender) {
			if (userModel.prefers) {
				return userModel.prefers.charAt(0) + userModel.gender.charAt(0);
			}
			else if (userModel.interested_in) {
				return userModel.interested_in[0].charAt(0) + userModel.gender.charAt(0);
			}
			else {
				return (userModel.gender.charAt(0) == "m" ? "f" : "m") + userModel.gender.charAt(0);
			}
		}
		return null;
	};
	
	
	function getFriendsGender() {
		if (userModel.orientation && userModel.orientation.length == 2) {
			return userModel.orientation.charAt(1) == "m" ? "f" : "m";
		}
		else if (userModel.gender) {
			if (userModel.prefers) {
				return userModel.prefers.charAt(0) == "m" ? "f" : "m";
			}
			else if (userModel.interested_in) {
				return userModel.interested_in[0].charAt(0) == "m" ? "f" : "m";
			}
			else {
				return (userModel.gender.charAt(0) == "m" ? "f" : "m");
			}
		}
		return "f";
	};


	$scope.getUsers = function(params) {

		var userParams = { chatroom: $stateParams.chatroom, category: $stateParams.category, activity: $stateParams.activity,  longitude: userModel.longitude, latitude: userModel.latitude, userId: { $ne: $scope.userModel.userId } };
		if ($stateParams.chatroom == "romance")
			userParams.orientation = getMatchOrientation();
		else if ($stateParams.chatroom == "friends")
			userParams.gender = getFriendsGender();
		userParams.indexLimit = _indexLimitUsers;

		if (params && params.isMore) {
			userParams.weight = { $lte: lastWeight };
			if (lastTimestamp) {
				userParams.timestamp = { $lte: lastTimestamp };
			}
		}
		else {
			$scope.isShowMoreUsers = false;
			$scope.isNoOneNearby = false;
			lastWeight = 0;
			lastTimestamp = null;
			clearUsers();
		}

		$timeout(function() { onGetUsers(userParams); }, 200 * indexStartUsers + 200);
	};


	function onGetUsers(userParams) {
		console.log("getUsers", JSON.stringify(userParams));

		MeetService.getUsers(userParams)
		.then(
			function(data) {
				//console.log("getUsers:", JSON.stringify(data));
				$scope.isNoOneNearby = ($scope.users.length == 0 && (!data.data || data.data.length == 0 || (data.data.length == 1 && data.data[0].userId == userModel.userId)));
				if (data.data) {
					appendUsers(data.data);
					if (data.data.length > 0) {
						lastWeight = data.data[data.data.length - 1].weight;
						lastTimestamp = data.data[data.data.length - 1].timestamp;
					}
				}
				$scope.isShowMoreUsers = !(data.data.length == 0 || data.data.length < _indexLimitUsers);
				$scope.$broadcast('scroll.refreshComplete');
				$scope.$broadcast('scroll.infiniteScrollComplete');
				if (data.data && data.data.length > 0 && !$scope.isNoOneNearby)
					getRecentLikes();
			},
			function(error) {
				//console.log("getUsers error:", JSON.stringify(error));
				$scope.isShowMoreUsers = false;
				$scope.isNoOneNearby = true;
				$scope.$broadcast('scroll.refreshComplete');
				$scope.$broadcast('scroll.infiniteScrollComplete');
			}
		);
	};


	function getRecentLikes() {
		MeetService.getRecentLikes()
		.then(
			function(data) {
				//console.log("getRecentLikes:", JSON.stringify(data));
				if (!data || !data.data || data.data.length == 0)
						return;
				for (i = 0; i < data.data.length; i++) {
					if (!data.data[i].likesUserId)
						continue;
					likeUser({ userId: data.data[i].likesUserId });
				}
			}
		);
	};


	// open user modal for profile / like / invite

	$scope.openProfile = function(user, isLikeUser, isAccept) {
    	closeDrawer();

		// skip opening profile if user's profile already open
		if ($scope.profileUserModal && $scope.profileUserModel && user.userId == $scope.profileUserModel.userId) {
			return;
		}


		// close currently opened profile
		$scope.closeProfile();


		// show vip profile if user is vip
		if (user.type == "p" && $scope.userModel.userId != user.userId) {
			$ionicModal.fromTemplateUrl('views/profile-vip.html', {
				scope: $scope,
				animation: 'slide-in-up'
			}).then(function(modal) {
				$scope.profileUserModal = modal;
				var params = { userId: user.userId };
				UserService.getUser(params).then(
					function(response) {
						$scope.profileUserModel = response.data;
						$scope.profileUserModal.show();

						if (ionic.Platform.isAndroid()) {
							$ionicPlatform.onHardwareBackButton(function() {
								$scope.closeProfile();
							});
						}
					}
				);
			});
		
			return;
		}


		// inviting user via phone number
		if (!user.userId && !user.phoneNumber)
			return;

		if (!user.userId && user.phoneNumber) {
			//console.log("inviteFriend", user);
			UserService.inviteFriend(user,
				function(response) {
					if (response && response.r == 's') {
						user.isInvited = true;
					}
				}
			);
			return;
		}
		

		// show profile
		$ionicModal.fromTemplateUrl('views/profile.html', {
			scope: $scope,
			animation: 'slide-in-up'
		}).then(function(modal) {
			$scope.profileUserModal = modal;
			UserService.getUser({ userId: user.userId }).then(
				function(response) {
					$scope.profileUserModel = response.data;
					$scope.profileUserModel.isLikeUser = isLikeUser;
					$scope.profileUserModel.isAccept = isAccept;
					$scope.isViewLongBio = false;
					$scope.bio = $scope.getBio();
					$scope.profileUserModal.show();

					if (ionic.Platform.isAndroid()) {
						$ionicPlatform.onHardwareBackButton(function() {
							$scope.closeProfile();
						});
					}

					//console.log($scope.profileUserModel);
				}
			);
			
			// get mutual friends if not self
			if (user.userId != userModel.userId) {
				UserService.getMutualFriends({ userId: user.userId }).then(
					function(response) {
						$scope.mutualFriends = response.data && response.data.length > 0
							? response.data
							: null;
					}
				);
			}
		});

		// viewed other user's profile
		if (isLikeUser && user.userId != $scope.userModel.userId) {
			MeetService.updateLike({ userId: user.userId, likesUserId: $scope.userModel.userId, isViewed: true });
		}

		ga.trackEvent('profileView', 'openProfile', 'userId', userModel.userId);
	};


	$scope.closeProfile = function() {
		if ($scope.profileUserModal) {
			$scope.isViewLongBio = false;
			$scope.bio = null;
			$scope.profileUserModal.remove();
			$scope.profileUserModal = null;
		}
	};


	$scope.editProfile = function() {
		$scope.closeProfile();
		$timeout(function() { $state.go('profile'); }, 100 );
	};


	$scope.editMedia = function(media) {
		//console.log("editMedia");
		AddMediaService.open($scope, media);
	}
	

	$scope.addMedia = function(mediaData) {
		AddMediaService.addMedia(mediaData, $scope);
	};

	
	function isOpen() {
		var hours = (new Date()).getHours();
		return !_openHours[$scope.userModel.activity] || (hours >= _openHours[$scope.userModel.activity].hourStart && hours < _openHours[$scope.userModel.activity].hourEnd);
	}


	$scope.likeUser = function(isLike, isAccept) {
		_urlScheme = null;
		$scope.closeProfile();
		if (isLike) {
			if (isAccept) {
				MeetService.acceptLike({ userId: $scope.userModel.userId, likesUserId: $scope.profileUserModel.userId });
				ga.trackEvent('likes', 'accept');
			}
			else {

				if (!isOpen()) {
					goChat();
					return;
				}
			
				MeetService.addLike({ userId: $scope.userModel.userId, likesUserId: $scope.profileUserModel.userId, chatroom: $scope.userModel.chatroom, activity: $scope.userModel.activity });
				ga.trackEvent('likes', 'like');
			}
			likeUser($scope.profileUserModel);
			playSound("woosh");
			
			// prompt to confirm phone number if not confirmed yet
			if (!_isShownConfirmPhoneNumber && !$scope.userModel.phoneNumber) {
				$scope.openConfirmPhoneNumberModal();
			}
		}
		else {
			if (isAccept) {
				MeetService.deleteLike({ userId: $scope.userModel.userId, likesUserId: $scope.profileUserModel.userId });
				ga.trackEvent('likes', 'decline');
			}
			else {
				MeetService.passLike({ userId: $scope.userModel.userId, likesUserId: $scope.profileUserModel.userId, chatroom: $scope.userModel.chatroom, activity: $scope.userModel.activity });
				ga.trackEvent('likes', 'pass');
			}
		}
	};


	function rainCheckLike() {
		_urlScheme = null;
		$scope.closeProfile();
		MeetService.rainCheckLike({ userId: $scope.userModel.userId, likesUserId: $scope.profileUserModel.userId });
		ga.trackEvent('likes', 'raincheck');
	};


	$scope.showLikeOptions = function(params) {
		var messages = [
			{ text: "Can we push it by an hour?" },
			{ text: "Let's meet another time" }
		];
		$ionicActionSheet.show({
			buttons: messages,
			buttonClicked: function(index) {
				switch(index) {
					case 0:
						// Can we push it by an hour?
						sendMessage(messages[0].text, { isConfirm: true });
						break;
					case 1:
						// Let's meet another time
						rainCheckLike();
						break;
				}
				ga.trackEvent('actionSheetLikeOption', index, 'userId', userModel.userId);
				return true;
			},
			cancelText: "close"
		});
	};


	// meet

	$scope.openMeet = function() {

		// skip opening is already open

		if ($scope.meetUserModal) {
			return;
		}

		$ionicModal.fromTemplateUrl('views/meet.html', {
			scope: $scope,
			animation: 'slide-in-up'
		}).then(function(modal) {
			$scope.meetUserModal = modal;
			$scope.meetUserModal.show();
		});
	};


	function refreshMeet() {
		if ($scope.meetUserModel && $scope.meetUserModel.meetId) {
			getMeet({ meetId : $scope.meetUserModel.meetId });
		}
	};


	function clearMeet() {
		$scope.isCancelingMeet = false;
		$scope.isRainCheckingMeet = false;
		$scope.isSelfConfirmed = false;
		$scope.isMeetConfirmed = false;
		$scope.isSelfCheckedIn = false;
		$scope.isMeetCheckedIn = false;
		$scope.meetUserModel = {};
		if (timerMeet) {
			clearInterval(timerMeet);
			timerMeet = null;
		}
	};


	$scope.closeMeet = function() {
		if ($scope.meetUserModal)
			$scope.meetUserModal.remove();
		$scope.meetUserModal = null;
	};


	function setMeet(data) {
		$scope.meetUserModel = data;

		setLocation(data);

		$scope.openMeet();
	};


	function setLocation(data) {
		if (!data.place || !data.place.name || !data.place.location)
			return "";
		$scope.placeName = data.place.name;

		var address = "";
		if (data.place.location.address) {
			var array = [].concat(data.place.location.address);
			address += array.join(", ");
		}
		if (data.place.location.city) {
			address += ", " + data.place.location.city;
		}
		$scope.placeAddress = address;
		
		$scope.isHappyHour = !!data.place.isHappyHour;
		$scope.isBrunch = !!data.place.isBrunch;
	};


	// confirm

	$scope.confirmMeet = function() {
		//console.log("confirmMeet");

		$scope.isSelfConfirmed = true;

		MeetService.confirmMeet({ userId: $scope.userModel.userId, meetUserId: $scope.meetUserModel.userId, meetId: $scope.meetUserModel.meetId });

		playSound("woosh");

		ga.trackEvent('meetView', 'confirmMeet', 'userId', userModel.userId);
		
		if ($scope.isHappyHour || $scope.isBrunch) {
			var textActivity = "";
			if ($scope.isHappyHour)
				textActivity = " for Happy Hour";
			if ($scope.isBrunch)
				textActivity = " for Brunch";
			showAlert("Don't forget to show", "this screen to your server when you arrive" + textActivity);
		}
	};


	// check in

	$scope.checkInMeet = function() {
		//console.log("checkInMeet");

		$scope.isSelfCheckedIn = true;

		MeetService.checkInMeet({ userId: $scope.userModel.userId, meetUserId: $scope.meetUserModel.userId, meetId: $scope.meetUserModel.meetId });

		playSound("woosh");

		ga.trackEvent('meetView', 'checkInMeet', 'userId', userModel.userId);
	};


	// canceling meet

	$scope.cancelingMeet = function() {
		$scope.isCancelingMeet = true;
		ga.trackEvent('meetView', 'cancellingMeet', 'userId', userModel.userId);
	};

	// rainChecking meet

	$scope.rainCheckingMeet = function() {
		$scope.isRainCheckingMeet = true;
		ga.trackEvent('meetView', 'rainCheckingMeet', 'userId', userModel.userId);
	};


	// cancel meet

	$scope.cancelMeet = function() {
		//console.log("cancelMeet");

		MeetService.cancelMeet({ userId: $scope.userModel.userId, meetUserId: $scope.meetUserModel.userId, meetId: $scope.meetUserModel.meetId });

		resetIsInvited($scope.meetUserModel);

		$scope.closeMeet();
		clearMeet();
		ga.trackEvent('meetView', 'cancelMeet', 'userId', userModel.userId);
	};

	// rain check meet

	$scope.rainCheckMeet = function() {
		//console.log("rainCheckMeet");

		MeetService.cancelMeet({ userId: $scope.userModel.userId, meetUserId: $scope.meetUserModel.userId, meetId: $scope.meetUserModel.meetId, isRainCheck: true });

		resetIsInvited($scope.meetUserModel);

		$scope.closeMeet();
		clearMeet();
		ga.trackEvent('meetView', 'rainCheckMeet', 'userId', userModel.userId);
	};


	// on cancel meet

	function onCancelMeet(data) {
		//console.log("event:meet:cancel", data);
		showMessage(data);
		$scope.closeMeet();
		clearMeet();
	};


	// uncancel meet

	$scope.uncancelMeet = function() {
		$scope.isCancelingMeet = false;
		ga.trackEvent('meetView', 'uncancelMeet', 'userId', userModel.userId);
	};

	// unrain check meet

	$scope.unrainCheckMeet = function() {
		$scope.isRainCheckingMeet = false;
		ga.trackEvent('meetView', 'unrainCheckMeet', 'userId', userModel.userId);
	};

	// done meet

	$scope.doneMeet = function() {
		//console.log("doneMeet");

		MeetService.doneMeet({ userId: $scope.userModel.userId, meetUserId: $scope.meetUserModel.userId, meetId: $scope.meetUserModel.meetId });

		$scope.closeMeet();
		clearMeet();

		$scope.showReviewPlace({ meetId: $scope.meetUserModel.meetId, place: { name: $scope.placeName, address: $scope.placeAddress }});

		ga.trackEvent('meetView', 'doneMeet', 'userId', userModel.userId);
	};


	function showAlert(name, text) {
		if ('cordova' in window && navigator.notification) {
			navigator.notification.alert(text, null, name);
		}
		else {
			if (alertPopup)
				alertPopup.close();

			$timeout(function() {
				alertPopup = $ionicPopup.alert({
					title: name,
					template: '<div class="center">' + text + '</div>',
					cssClass: 'alert'
				});
			}, 500);
		}
	};


	$scope.openMap = function() {
		if ($scope.placeAddress) {
			if (window.cordova) {
				if (ionic.Platform.isIOS()) {
					window.location.href = "maps://?daddr=" + $scope.placeAddress;
				}
				else {
					window.open("google.navigation:q=" + $scope.placeAddress + "&mode=d", "_system");
				}
			}
			else {
				window.open("http://maps.google.com/?saddr=Current+Location&daddr=" + $scope.placeName + ", " + $scope.placeAddress, "map");
			}
		}
		ga.trackEvent('meetModal', 'openMap', 'userId', userModel.userId);
	};


	// messages

	function setMessageBadgeNumber(badgeNumber) {
		$scope.badgeNumber = badgeNumber;
		$scope.isNewMessage = badgeNumber > 0;
	}
	
	
	function checkNewMessage() {
		UserService.getMessages({ indexLimit: 1 }).then(
			function(response) {
				if (response.data && response.data.length > 0) {
					var lastMessage = response.data[0];
					if (!$scope.userModel.lastMessageId || lastMessage.messageId != $scope.userModel.lastMessageId) {
						setMessageBadgeNumber(1);
						UserService.update({ lastMessageId: lastMessage.messageId });
					}
				}
			}
		);
	}


	$scope.showMessages = function() {
		if ($scope.messagesModal)
			$scope.closeMessages();

		setMessageBadgeNumber(0);

		$scope.getMessages();

		$ionicModal.fromTemplateUrl('views/messages.html', {
			scope: $scope,
			animation: 'slide-in-up'
		}).then(function(modal) {
			$scope.messagesModal = modal;
			$scope.messagesModal.show();
			if (ionic.Platform.isAndroid()) {
				$ionicPlatform.onHardwareBackButton(function() {
					$scope.closeMessages();
				});
			}
		});
	}
	
	
	$scope.closeMessages = function() {
		if ($scope.messagesModal) {
			$scope.messagesModal.remove();
			$scope.messagesModal = null;
			$scope.messages = [];
		}
	};
	
	
	$scope.getMessages = function(paramsIn) {
		var params = { indexLimit: _indexLimitMessages };

		if (paramsIn && paramsIn.isMore) {
			params.indexStart = $scope.messages.length;
		}
		else {
			$scope.isShowMoreMessages = false;
			$scope.messages = [];
		}
		
		UserService.getMessages(params).then(
			function(response) {
				var numMessages = (response.data && response.data.length > 0) ? response.data.length : 0;
				if (!$scope.messages)
					$scope.messages = [];
				if (numMessages > 0) {
					for (i = 0; i < numMessages; i++) {
						$scope.messages.push(response.data[i]);
					}
				}
				$scope.isShowMoreMessages = !(numMessages == 0 || numMessages < _indexLimitMessages);
				$scope.$broadcast('scroll.refreshComplete');
				$scope.$broadcast('scroll.infiniteScrollComplete');
			},
			function(error) {
				$scope.isShowMoreMessages = false;
				$scope.$broadcast('scroll.refreshComplete');
				$scope.$broadcast('scroll.infiniteScrollComplete');
			}
		);
	};
	

	// confirm phone number

	$scope.openConfirmPhoneNumberModal = function() {
		_isShownConfirmPhoneNumber = true;
		$scope.isPhoneCodeSent = false;
		$scope.isPhoneCodeError = false;
		$scope.isPhoneCodeEdit = $scope.userModel.phoneNumber ? false : true;
		$scope.isPhoneNumberConfirmed = false;
		$scope.closeConfirmPhoneNumberModal();
		$scope.phoneModel.isPushText = $scope.userModel.hasOwnProperty("isPushText") ? $scope.userModel.isPushText : true;
		$ionicModal.fromTemplateUrl('views/confirm-phone.html', {
			scope: $scope,
			animation: 'slide-in-up'
		}).then(function(modal) {
			$scope.confirmPhoneNumberModal = modal;
			$scope.confirmPhoneNumberModal.show();
		});
	};


	$scope.closeConfirmPhoneNumberModal = function() {
		if ($scope.confirmPhoneNumberModal)
			$scope.confirmPhoneNumberModal.remove();
	};


	$scope.updateUserIsPushText = function(isPushText) {
		UserService.update({ isPushText: isPushText });
		$scope.userModel.isPushText = isPushText;
	};


	$scope.phoneUser = function() {
		delete $scope.userModel.phoneCode;
		delete $scope.phoneModel.phoneCode;
		$scope.isPhoneCodeSent = false;
		$scope.isPhoneCodeError = false;
		$scope.isPhoneCodeEdit = false;
		$scope.isPhoneNumberConfirmed = false;
		if (!$scope.userModel.phoneUnconfirmed || String($scope.userModel.phoneUnconfirmed).length < 10)
			return;

		//console.log('phoneUser: ' + $scope.userModel.phoneUnconfirmed + ' ' + String($scope.userModel.phoneUnconfirmed).length);

		var params = { phoneUnconfirmed: String($scope.userModel.phoneUnconfirmed) };
		UserService.phoneUser(params,
			function(response) {
				if (response.r && response.r == 's')
					$scope.isPhoneCodeSent = true;
			}
		);
	};


	$scope.confirmUser = function() {
		if (!$scope.phoneModel.phoneCode || String($scope.phoneModel.phoneCode).length < 5)
			return;

		var params = { phoneCode: String($scope.phoneModel.phoneCode) };
		if ($scope.phoneModel.isPushText) {
			params.isPushText = true;
		}
		UserService.confirmUser(params,
			function(response) {
				if (response.r && response.r == 's') {
					$scope.isPhoneNumberConfirmed = true;
					$timeout(
						function() {
							$scope.closeConfirmPhoneNumberModal();
						},
						2000
					);					
				}
				else {
					$scope.isPhoneCodeError = true;
				}
			}
		);
	};


	// faq

	$scope.showFaq = function() {
		$cordovaInAppBrowser.open(AppConfig.urlFAQ + '?_=' + (new Date()).getTime(), 'jones', { allowInlineMediaPlayback: "yes" });
		ga.trackEvent('faq', 'main', 'userId', userModel.userId);
		closeDrawer();
	};


	// feedback

	$scope.giveFeedback = function() {
		$cordovaEmailComposer.isAvailable().then(
			function() {
				var email = {
					to: 'support@joinjones.com',
					subject: 'Feedback for JONES v' + _appVersion + ' ' + ($scope.userModel && $scope.userModel.userId ? $scope.userModel.userId : ''),
					body: '',
					isHtml: false
				};

				$cordovaEmailComposer.open(email);
			},
			function () {
				// default email not set up or not available
				window.open('mailto:support@joinjones.com?subject=Feedback for JONES v' + _appVersion + ' ' + ($scope.userModel && $scope.userModel.userId ? $scope.userModel.userId : ''), '_system');
			}
		);
		ga.trackEvent('feedback', 'main', 'userId', userModel.userId);
		closeDrawer();
	}


	$scope.tellFriends = function() {
		var appLink = ionic.Platform.isAndroid() ? AppConfig.urlApp.android : AppConfig.urlApp.ios;
		$cordovaSocialSharing.share("Join me for drinks & socializing on JONES app", appLink, null, appLink);
		ga.trackEvent('tellFriends', 'chatroom', 'userId', userModel.userId);
		closeDrawer();
	};


	$scope.signOut = function() {
		confirmLogout();
		closeDrawer();
	}


	function closeDrawer() {
		$timeout(function() { $scope.closeDrawer(); }, 300);
	}


	function confirmLogout() {
		if (!navigator.notification)
			return;

		navigator.notification.confirm(
			"",
			function(index) {
				switch(index) {
					case 2:
						var userId = userModel.userId;
						logout();
						ga.trackEvent('logout', 'chatroom', 'userId', userId);
				}
			},
			"Sign Out?",
			["No", "Yes"]
		);
	};


	$scope.showFlag = function() {
		$ionicActionSheet.show({
			buttons: [
				{ text: "flag as inapprioriate" },
				{ text: "" }
			],
			buttonClicked: function(index) {
				switch (index) {
					case 0:
						UserService.flag({ flagUserId: $scope.profileUserModel.userId });
						return true;
					case 1:
						return false;
				}
				return true;
			},
			destructiveText: "block",
			destructiveButtonClicked: function() {
				navigator.notification.confirm(
					$scope.getFirstName($scope.profileUserModel.name) + "?",
					function(index) {
						switch(index) {
							case 2:
								UserService.block({ blockUserId: $scope.profileUserModel.userId });
						}
					},
					"Block",
					["Cancel", "Block"]
				);
				return true;
			},
			cancelText: "close"
		});
	};


	$scope.showReviewPlace = function(params) {
		$ionicActionSheet.show({
			titleText: "rate this venue",
			buttons: [
				{ text: "love it!" },
				{ text: "meh" },
				{ text: "sucks" }
			],
			buttonClicked: function(index) {
				switch(index) {
					case 0:
						params.rating = 5;
						break;
					case 1:
						params.rating = 3;
						break;
					case 2:
						params.rating = 1;
						break;
				}
				MeetService.reviewPlace(params);
				ga.trackEvent('actionSheetReview', params.rating, 'userId', userModel.userId);
				return true;
			},
			cancelText: "close"
		});
	};


	var sendMessagesTextArray = ["I'll be there!", "I'm on my way", "I'm at the entrance", "I'm at the bar", "10 mins away", "20 mins away", "30 mins away"];
	var sendMessagesButtonsArray = [];
	for (i = 0; i < sendMessagesTextArray.length; i++) {
		sendMessagesButtonsArray.push({ text: sendMessagesTextArray[i] });
	}


	$scope.showSendMessage = function() {
		$ionicActionSheet.show({
			buttons: sendMessagesButtonsArray,
			cancelText: "close",
			buttonClicked: function(index) {
				var isConfirm = sendMessagesTextArray[index].indexOf("?") == sendMessagesTextArray[index].length - 1;
				var data = {};
				if (isConfirm) {
					data.isConfirm = true;
				}
				if (sendMessagesTextArray[index] == "Yes") {
					data.hours = 2;
				}
				$timeout(function() { sendMessage(sendMessagesTextArray[index], data); }, 100);
				return true;
			}
		});

		ga.trackEvent('meetView', 'showSendMessage', 'userId', userModel.userId);
	};


	function sendMessage(text, data) {
		//console.log("meetUserModel", $scope.meetUserModel);
		var message = data;
		message.text = text;
		if ($scope.meetUserModel && $scope.meetUserModel.meetId) {
			message.meetId = $scope.meetUserModel.meetId;
		}
		else if ($scope.profileUserModel && $scope.profileUserModel.userId) {
			message.meetUserId = $scope.profileUserModel.userId;
		}
		MeetService.sendMessage(message);
		playSound("woosh");
	};


	function setGeoLocation() {
		if (AppConfig.isSimulator) {
			userModel.latitude = AppConfig.test.latitude;
			userModel.longitude = AppConfig.test.longitude;
			onGetGeoLocation({ coords: { latitude: userModel.latitude, longitude: userModel.longitude }});
		}
		else if (navigator.geolocation) {
			navigator.geolocation.getCurrentPosition(onGetGeoLocation);
		}
	};


	function onGetGeoLocation(geoLocation) {
		//console.log("onGetGeoLocation");
		if (userModel.latitude != geoLocation.coords.latitude || userModel.longitude != geoLocation.coords.longitude) {
			userModel.latitude = geoLocation.coords.latitude;
			userModel.longitude = geoLocation.coords.longitude;
			UserService.update({ latitude: userModel.latitude, longitude: userModel.longitude });

			if (timerGeo)
				clearInterval(timerGeo);
			timerGeo = setInterval(setGeoLocation, 60000);
		}
	};


	function logout() {
		_isInitPush = false;
		UserService.logout();
		goHome();
	};


	function disconnect() {
		if (_socket) {
			//console.log("disconnect _socket");
			_socket.disconnect();
			_socket = null;
		}
	};


	function goHome() {
		disconnect();
		$ionicHistory.nextViewOptions({
			disableBack: true
		});
		$timeout(function() { $state.go('home', { reload: true, cache: false }) }, 500);
	};


	function goChat() {
		$timeout(function() { $state.go('chat'); }, 100);
	};
	

	function checkMeet() {
		//console.log("checkMeet _urlScheme", _urlScheme);
		setGeoLocation();
		
		checkNewMessage();
		
		if (_urlScheme) {
			var params = getParamsObjFromUrl(_urlScheme);
			_urlScheme = null;
			//console.log("params", JSON.stringify(params));
			if (params) {
				if (params.meetId) {
					getMeet(params);
				}
				else if (params.likeUserId) {
					processLike(params);
				}
				return;
			}
		}

		getMeet({ userId: $scope.userModel.userId });
	};


	function getMeet(params) {
		MeetService.getMeet(params)
		.then(
			function(response) {
				//console.log("getMeet", JSON.stringify(response));
				if (response.r && response.r == "s" && response.data) {
			  			processMeet(response.data);
				}
				else {
					if ($scope.meetUserModal && $scope.meetUserModel) {
						if (!$scope.isSelfCheckedIn && !$scope.isMeetCheckedIn) {
							MeetService.getMyMeets({ meetId: $scope.meetUserModel.meetId, indexLimit: 1 })
							.then(
								function(response) {
									var data = { name: $scope.meetUserModel.name };
									if (response.data && response.data.length && response.data[0].meet) {
										if (response.data[0].meet.isRainCheck) {
											data.text = "Let's meet another time!";
										}
										else if (response.data[0].meet.isCancel) {
											data.text = "has canceled";
										}
										if (response.data[0].meet.isRainCheck || response.data[0].meet.isCancel) {
											onCancelMeet(data);
										}
									}
								},
								function(error) {
									onCancelMeet();
								}
							);
						}
					}

					var params = { indexLimit: 1 };
					MeetService.getWhoLikesMe(params)
					.then(
						function(response) {
							if (response.r && response.r == "s") {
								//console.log("getWhoLikesMe", JSON.stringify(response));
								if (response.data && response.data.length > 0) {
									var data = response.data[0];
									if (!data.isViewed) {
										MeetService.updateLike({ likeId: data.likeId, isViewed: true })
										.then(
											function(response) {
												var paramLike = { likeUserId: data.userId };
												processLike(paramLike);
											}
										);
									}
								}
							}
						}
					);
				}
			}
		);
	};


	function processMeet(meet) {
		if (!meet || !meet.meetId) {
			if ($scope.meetUserModal) {
				$scope.closeMeet();
				clearMeet();
			}
			checkLike();
			return;
		}
		var users = meet.users;
		var meetUser;
		var selfUser;
		if (meet.users[0].userId != $scope.userModel.userId) {
			meetUser = meet.users[0];
			selfUser = meet.users[1];
		}
		else {
			meetUser = meet.users[1];
			selfUser = meet.users[0];
		}
		meetUser.meetId = meet.meetId;
		meetUser.activity = meet.activity;
		meetUser.place = meet.place;
		meetUser.timestamp = meet.timestamp;
		if (selfUser.isSelfConfirmed) {
			$scope.isSelfConfirmed = true;
		}
		if (meetUser.isSelfConfirmed) {
			$scope.isMeetConfirmed = true;
		}
		if (selfUser.isSelfCheckedIn) {
			$scope.isSelfCheckedIn = true;
		}
		if (meetUser.isSelfCheckedIn) {
			$scope.isMeetCheckedIn = true;
		}
		console.log("meetUser", JSON.stringify(meetUser));
		setMeet(meetUser);
	};


	function checkLike() {
		var params = { indexLimit: 1 };
		MeetService.getWhoLikesMe(params)
		.then(
			function(response) {
				//console.log("getWhoLikesMe", JSON.stringify(response));
				if (response.r && response.r == "s") {
					if (response.data && response.data.length > 0) {
						var data = response.data[0];
						if (!data.isViewed) {
							var paramLike = { likeUserId: data.userId };
							processLike(paramLike);
						}
					}
				}
			}
		);
	};


	function processLike(params) {
		var userParams = { userId: params.likeUserId };
		UserService.getUser(userParams)
		.then(
			function(data) {
				//console.log("processLike", JSON.stringify(data));
				if (data.r && data.r == "s" && data.data) {
					showInvitePrompt(data.data);
				}
			}
		);
	};


	function filterPhoneNumber(phoneNumber) {
        if (phoneNumber) {
                phoneNumber = phoneNumber.replace(/[^0-9\+]/g, '');
                if (phoneNumber.substring(0, 1) != "+" && phoneNumber.length == 10)
                        phoneNumber = "+1" + phoneNumber;
        }
		return phoneNumber;
	};


	function getContacts() {
		$scope.statusContacts = "loading...";
		$timeout(onGetContacts, 100);
	};


	function onGetContacts() {
		var options = {};
		options.filter = "";
		options.multiple = true;
		options.hasPhoneNumber = true;
		options.desiredFields = [
			navigator.contacts.fieldType.name,
			navigator.contacts.fieldType.phoneNumbers,
			navigator.contacts.fieldType.photos
		];
		$cordovaContacts.find(options).then(
			function(contacts) {
				//console.log(contacts);
				var numContacts = contacts.length;
				for (i = 0; i < numContacts; i++) {
					if (!contacts[i].phoneNumbers)
						continue;
					var contact = {};
					contact.id = contacts[i].id;
					contact.name = contacts[i].name.formatted;
					if (contacts[i].phoneNumbers)
						contact.phoneNumber = filterPhoneNumber(contacts[i].phoneNumbers[0].value);
					if (contacts[i].photos)
						contact.urlPhoto = contacts[i].photos[0].value;
					addContact(contact);
				}
				$scope.statusContacts = "";
			}
		);
	};


	$scope.pickContact = function() {
		$cordovaContacts.pickContact().then(
			function(contactPicked) {
				console.log(contactPicked);
				var contact = {};
				contact.id = contactPicked.id;
				contact.name = contactPicked.name.formatted;
				if (contactPicked.phoneNumbers)
					contact.phoneNumber = filterPhoneNumber(contactPicked.phoneNumbers[0].value);
				if (contactPicked.photos)
					contact.urlPhoto = contactPicked.photos[0].value;
				$scope.addFriend(contact);
			}
		);
	};


	$scope.addFriend = function(contact) {
		console.log("addFriend", contact);
		UserService.addFriend(contact,
			function(response) {
				if (response && response.r == 's') {
					removeContact(contact);
					if (response.data) {
						// contact already signed up with userId
						addUser(response.data);
					}
					else {
						// contact has not signed up with userId
						addUser(contact);
					}
				}
			}
		);
		removeContact(contact);
	};


	// open invite modal

	$scope.openInviteModal = function() {
		$ionicModal.fromTemplateUrl('views/invite.html', {
			scope: $scope,
			animation: 'slide-in-up'
		}).then(function(modal) {
			$scope.inviteModal = modal;
			clearContacts();
			getContacts();
			$scope.inviteModal.show();
		});
	};


	$scope.closeInviteModal = function() {
		clearContacts();
		$scope.inviteModal.remove();
	};


	// show invite prompt
	function showInvitePrompt(params) {

		_urlScheme = null;

		if ($scope.meetUserModal)
			return;

		var userId = (params.likeUserId ? params.likeUserId : params.userId);

		if (invites[userId]) {
			return;
		}
		else {
			invites[userId] = params;
		}

		if (!params.text) {
			var textSuffix = "";
			if (params.chatroom == "romance")
				textSuffix = " \u2764";
			params.text = $scope.getFirstName(params.name) + " " + (params.activity ? "invites you for " + params.activity : "sends you an invite") + textSuffix;
		}

		navigator.notification.confirm(
			params.text,
			function(index) {
				switch(index) {
					case 1:
						MeetService.updateLike({ userId: userId, likesUserId: $scope.userModel.userId, isViewed: true });
						delete invites[userId];
						break;
					case 2:
						$scope.openProfile({ userId: (params.likeUserId ? params.likeUserId : params.userId) }, true, true);
						delete invites[userId];
						break;
				}
			},
			"Invitation",
			["Close", "Open"]
		);
	};


	function playSound(soundName) {
		try {
			var soundFile = (ionic.Platform.isAndroid() ? "/android_asset/www/" : "") + "sounds/" + (soundName.indexOf(".") != -1 ? soundName : soundName + ".wav");
			var sound = new Media(soundFile);
			sound.play({ playAudioWhenScreenIsLocked: false });
		}
		catch (exception) {
			console.log(exception);
		}
	};


	$scope.openPokemonGo = function() {
		if (ionic.Platform.isIOS()) {
			window.open(_urlPokemoniOS);
		}
		else if (ionic.Platform.isAndroid()) {
			window.plugins.webintent.startActivity({
					action: window.plugins.webintent.ACTION_VIEW,
					url: _urlPokemonGooglePlayApp
				},
				function() { console.log('success'); },
				function(errorMsg) { console.log('error: ' + errorMsg); }
			);
		}
	};


	// socket events

	function initSocket() {
		if (_socket) {
			onConnectUser();
			return;
		}

		console.log("_socket connect");
		_socket = io.connect(AppConfig.urlServer, { 'force new connection': true });

		// on connect init
		_socket.on('event:connect:init', function(data) {
			connectUser();
		});

		// on new user
		_socket.on('event:new:user', function(data) {
			onNewUser(data);
		});

		// new like / invite
		_socket.on('event:like:new', function(data) {
			data.eventType = 'event:like:new';
			if (!data.sound)
				data.sound = "pop";
			processEvent(data);
		});
		
		// raincheck like / invite
		_socket.on('event:like:raincheck', function(data) {
			data.eventType = 'event:like:raincheck';
			if (!data.sound)
				data.sound = "woosh";
			processEvent(data);
		});

		// new meet
		_socket.on('event:meet:new', function(data) {
			console.log("event:meet:new", JSON.stringify(data));
			data.eventType = 'event:meet:new';
			if (!data.sound)
				data.sound = "pop";
			processEvent(data);
		});

		// confirmed
		_socket.on('event:meet:confirm', function(data) {
			console.log("event:meet:confirm", data);
			data.eventType = 'event:meet:confirm';
			if (!data.sound)
				data.sound = "pop";
			processEvent(data);
		});

		// checked in
		_socket.on('event:meet:checkin', function(data) {
			console.log("event:meet:checkin", data);
			data.eventType = 'event:meet:checkin';
			if (!data.sound)
				data.sound = "pop";
			processEvent(data);
		});

		// made other plans
		_socket.on('event:meet:other', function(data) {
			data.eventType = 'event:meet:other';
			if (!data.sound)
				data.sound = "pop";
			processEvent(data);
		});

		// canceled meet
		_socket.on('event:meet:cancel', function(data) {
			data.eventType = 'event:meet:cancel';
			if (!data.sound)
				data.sound = "pop";
			processEvent(data);
		});

		// new socket message
		_socket.on('event:new:message', function(data) {
			//console.log("event:new:message", JSON.stringify(data));
			data.eventType = 'event:new:message';
			if (!data.sound)
				data.sound = "woosh";
			processEvent(data);
		});
	};


	function processEvent(data) {
		if (data.eventId) {
			if (eventIds[data.eventId]) {
				return;
			}
			eventIds[data.eventId] = true;
		}

		if (data.sound)
			playSound(data.sound);
		
		switch(data.eventType) {
			// new like
			case 'event:like:new':
				showInvitePrompt(data);
				break;
				
			// raincheck like
			case 'event:like:raincheck':
				showAlert($scope.getFirstName(data.name), data.text);
				break;
		
			// new meet
			case 'event:meet:new':
				$scope.closeProfile();
				setMeet(data);
				break;

			// confirm
			case 'event:meet:confirm':
				$scope.$apply(function() { $scope.isMeetConfirmed = true; });
				showAlert($scope.getFirstName(data.name), data.text);
				break;
		
			// checkin
			case 'event:meet:checkin':
				$scope.$apply(function() { $scope.isMeetCheckedIn = true; });
				playSound(data.sound ? data.sound : "pop");
				showAlert($scope.getFirstName(data.name), data.text);
				break;
		
			// other
			case 'event:meet:other':
				showAlert($scope.getFirstName(data.name), data.text);
				break;
		
			// cancel
			case 'event:meet:cancel':
				onCancelMeet(data);
				break;
		
			// new message
			case 'event:new:message':
			default:
				showMessage(data);
				break;
		}
	}


	// process message
	function processMessage(data) {
		if (!data)
			return;
			
		if (data.isRefresh)
			refreshMeet();
	}
	

	// show message
	function showMessage(data) {
		if (data.isConfirm) {
			navigator.notification.confirm(
				data.text,
				function(index) {
					switch(index) {
						case 1:
							delete data.isConfirm;
							sendMessage("No", data);
							break;
						case 2:
							delete data.isConfirm;
							data.hours = 2;
							sendMessage("Yes", data);
							break;
					}
				},
				$scope.getFirstName(data.name),
				["No", "Yes"]
			);
		}
		else {
			if (data.name) {
				showAlert($scope.getFirstName(data.name), data.text);
			}
			else {
				showAlert("", data.text);
			}
		}

		processMessage(data);
	}


	$scope.openMessage = function(message) {
		if (!message) {
			return;
		}
		if (message.fromUserId != $scope.userModel.userId) {
			$scope.openProfile({ userId: message.fromUserId }, false, true);
		}
		else if (message.toUserId != $scope.userModel.userId) {
			$scope.openProfile({ userId: message.toUserId }, true, false);
		}
	}


	// events

	var offCheckMeet = $rootScope.$on('checkMeet', function(event, params) {
		checkMeet();
	});
	

	var offInvitePush = $rootScope.$on('invitePush', function(event, params) {
		showInvitePrompt(params);
	});
	
	
	var offMessagePush = $rootScope.$on('messagePush', function(event, params) {
		processEvent(params);
	});
	

	$scope.$on('$destroy', function() {
		console.log("destroy");
		offCheckMeet();
		offInvitePush();
		offMessagePush();

		$scope.closeProfile();
		$scope.closeMeet();
		clearMeet();
		clearUsers();
		$scope.users = null;
		currentChatroomUsers = null;

		if (timerGeo) {
			clearInterval(timerGeo);
			timerGeo = null;
		}

		if (timerMeet) {
			clearInterval(timerMeet);
			timerMeet = null;
		}

		disconnect();
	});


	// init

	function init() {
		if (!userModel) {
			disconnect();
			goHome();
		}
		else {
			UserService.syncUserModel($scope.userModel);
		
			console.log("connectUser $stateParams:", JSON.stringify($stateParams));
			if ($stateParams.chatroom)
				userModel.chatroom = $stateParams.chatroom;
			if ($stateParams.category)
				userModel.category = $stateParams.category;
			if ($stateParams.activity)
				userModel.activity = $stateParams.activity;
			initSocket();

			if (timerGeo) {
				clearInterval(timerGeo);
			}
			timerGeo = setInterval(setGeoLocation, 60000);

			if (timerMeet) {
				clearInterval(timerMeet);
			}
			timerMeet = setInterval(refreshMeet, 60000);

			checkAvailable();
		}

		ga.trackPageview('/chatroom');
	}


	init();
}]);
