// requires $scope.addMedia(url<String>, {type: String, title: String, description: String})

angular.module('hh')
.directive('instagramSelector', ['InstagramService', function(InstagramService) {
	return {
		restrict: 'E',
		replace: true,

		link: function(scope, element, attr) {
			if (typeof scope.addMedia !== 'function') {
				throw new Error('<instagram-photo-selector /> requires $scope to have' +
												'`addMedia()` function defined.');
			}
			
			scope.search = '';

			scope._selectMedia = function(o) {
				var type = o.videos ? "v" : "i";
				var url = type == "v" ? o.videos.standard_resolution.url :
					o.images.standard_resolution.url;

				scope.addMedia({ type: type, url: url, thumbnail: o.images.thumbnail.url });
			};

			scope.blur = function() {
				scope.loadMedia();
			}

			scope.media = [];
			var recentUser = InstagramService.getRecentUser();
			scope.search = recentUser ? recentUser.username : '';

			scope.loadMedia = function() {
				console.log("InstagramSelector loadMedia");

				if (!scope.search || scope.search == '') {
					scope.media = [];
					InstagramService.clearUser();
					return;
				}

				InstagramService.getRecentPhotos(scope.search.trim().replace("@", "")).then(
					function(media) { scope.media = media; },
					function(error) { }
				);
			};
			
			scope.clearMedia = function() {
				scope.search = '';
				scope.media = [];
				scope.addMedia();
			}

			// init
			scope.loadMedia();
		},

		template: [
			'<div>',
				'<form class="list">',
					'<div class="item item-input-inset bg-darker borderless">',
						'<label class="item-input-wrapper bg-transparent">',
							'<button class="button-clear icon ion-social-instagram-outline" ng-click="loadMedia()"></button>',
							'<input type="text" class="white" style="text-align:center" placeholder="Instagram username" ng-model="search" ng-blur="blur()" autocorrect="off" autocapitalize="off">',
						'</label>',
						'<button class="button button-clear button-small icon ion-close-round" ng-click="clearMedia()"></button>',
					'</div>',
				'</form>',
				'<ion-scroll direction="x" class="horizontal-scroll">',
					'<img "photo-container" ng-repeat="m in media" class="photos" ng-src="{{m.images.low_resolution.url}}" ng-click="selectMedia(m)" />',
				'</ion-scroll>',
			'</div>',
		].join('')
	};
}]);
