angular.module('hh')
.service('AddMediaService',
[
	'$ionicActionSheet',
	'$ionicLoading',
	'$timeout',
	'PhotoService',
	'VideoService',
	'UserService',
function(
	$ionicActionSheet,
	$ionicLoading,
	$timeout,
	PhotoService,
	VideoService,
	UserService
) {

	var options = [
		'instagramEnabled',
	];

	// disable all media options
	function close(_scope) {
		options.forEach(function(o) {
			_scope[o] = false;
		});
	}


	function open(_scope, mediaData) {
	
		var isDelete = (_scope.userModel.media && _scope.userModel.media.length > 1);
	
		var buttons = [
			{
				text: 'video selfie',
				onTap: function() {
					VideoService.open(_scope);
					return true;
				}
			},
			{
				text: 'photo selfie',
				onTap: function() {
					PhotoService.open(_scope, false);
					return true;
				}
			},
			{
				text: 'photo album',
				onTap: function() {
					PhotoService.open(_scope, true);
					return true;
				}
			},
			{
				text: 'Instagram',
				onTap: function() {
					close(_scope);
					_scope.instagramEnabled = true;
					return true;
				}
			}
		];
 
		if (mediaData && mediaData.type == "i") {
			buttons.push(
				{
					text: 'set as profile photo',
					onTap: function() {
						_scope.userModel.urlPhoto = mediaData.thumbnail;
						for (i = 0; i < _scope.userModel.media.length; i++) {
							if (_scope.userModel.media[i].url == mediaData.url) {
								_scope.userModel.media.move(i, 0);
								break;
							}
						}
						UserService.update(_scope.userModel);
						return true;
					}
				}
			);
		}
 
		var sheet = {
			buttons: buttons,
			buttonClicked: function(index) {
				if (index < buttons.length && buttons[index] && buttons[index].onTap)
					buttons[index].onTap();
				return true;
			},
			cancelText: "close"
		}

		if (mediaData && isDelete) {
			sheet.destructiveText = "delete this " + (mediaData.type == "v" ? "video" : "photo");
			sheet.destructiveButtonClicked = function() {
				confirmDelete(mediaData, _scope);
				return true;
			}
		}

		$ionicActionSheet.show(sheet);
	};
	
	
	function confirmDelete(mediaData, _scope) {
		navigator.notification.confirm(
			"",
			function(index) {
				switch(index) {
					case 2:
						deleteMedia(mediaData, _scope);
				}
				return true;
			},
			"Delete this photo?",
			["No", "Yes"]
		);
	}
	
	
	function deleteMedia(mediaData, _scope) {
		//console.log("deleteMedia metaData", JSON.stringify(mediaData));
		_scope.$apply(function () {
			_scope.userModel.media = _scope.userModel.media.filter(function(item) {
				return item.url !== mediaData.url;
			});
		});
		var isProfilePhotoDeleted = true;
		var numPhotos = _scope.userModel.media.length;
		for (i = 0; i < numPhotos; i++) {
			if (_scope.userModel.urlPhoto == _scope.userModel.media[i].thumbnail) {
				isProfilePhotoDeleted = false;
				break;
			}
		}
		if (isProfilePhotoDeleted) {
			_scope.userModel.urlPhoto = _scope.userModel.media[0].thumbnail;
		}
		UserService.update(_scope.userModel);

		if (_scope.profileUserModel && _scope.profileUserModel.userId == _scope.userModel.userId) {
			_scope.profileUserModel.media = _scope.userModel.media;
		}
	}
	
	
	function addMedia(mediaData, _scope) {
		close(_scope);
		if (!mediaData) {
			$ionicLoading.hide();
			return;
		}

		console.log("addMedia mediaData", JSON.stringify(mediaData))

		$timeout(function() {
			if (!_scope.userModel.media) {
				_scope.userModel.media = [];
			}
			_scope.$apply(function() { _scope.userModel.media.unshift(mediaData); });

			console.log("userModel", JSON.stringify(_scope.userModel));

			UserService.update(_scope.userModel);

			if (_scope.profileUserModel && _scope.profileUserModel.userId == _scope.userModel.userId) {
				_scope.profileUserModel.media = _scope.userModel.media;
			}

			$ionicLoading.hide();

		}, 500);
	}
	
	this.open = open;
	this.close = close;
	this.addMedia = addMedia;
	this.deleteMedia = deleteMedia;
}]);
