'use strict';

angular.module('hh').factory('FacebookService', [
	'$q',
	'$cordovaFacebook',
	'UserService',
	'LocalStorageService',
function(
	$q,
	$cordovaFacebook,
	UserService,
	LocalStorageService
) {
	
	function login() {

		logout();
	
		var deferred = $q.defer();

		if ('cordova' in window) {
		$cordovaFacebook.login(["public_profile,email,user_birthday,user_relationships,user_relationship_details,user_work_history,user_education_history,user_about_me,user_photos,user_friends"])
			.then(
				getProfile,
				function(error) {
					console.log("FacebookLoginService login error",  error);
					deferred.reject(error);
				}
			)
			.then(
				getProfilePhotos,
				function(error) {
					console.log("FacebookLoginService getProfile error",  error);
					deferred.reject(error);
				}
			)
			.then(
				getFriends,
				function(error) {
					console.log("FacebookLoginService getProfilePhotos error",  error);
					deferred.reject(error);
				}
			)
			.then(
				UserService.createSession,
				function(error) {
					console.log("FacebookLoginService getFriends error",  error);
					deferred.reject(error);
				}
			)
			.then(
				function(data) {
					deferred.resolve(data);
				},
				function(error) {
					console.log("FacebookLoginService createSession error",  error);
					deferred.reject(error);
				}
			);
		}
		else {

			UserService.oauth('facebook', ["public_profile,email,user_birthday,user_relationship_details,user_work_history,user_education_history,user_about_me,user_photos,user_friends"])
			.then(
				getProfile,
				function(error) {
					console.log("FacebookLoginService error oauth",  error);
					deferred.reject(error);
				}
			)
			.then(
				getProfilePhotos,
				function(error) {
					console.log("FacebookLoginService error getProfile",  error);
					deferred.reject(error);
				}
			)
			.then(
				UserService.createSession,
				function(error) {
					console.log("FacebookLoginService error getProfilePhotos",  error);
					deferred.reject(error);
				}
			)
			.then(
				function(data) {
					deferred.resolve(data);
				},
				function(error) {
					console.log("FacebookLoginService error createSession",  error);
					deferred.reject(error);
				}
			);
		}

		return deferred.promise;
	}
	

	function logout() {
		UserService.clearUserModel();

		if ('cordova' in window) {
			$cordovaFacebook.logout();
		}
		else {
			hello('facebook').logout();
		}
	}
	

	function getProfile(data) {
		var deferred = $q.defer();

		console.log("FacebookService.login:", JSON.stringify(data));
		
		if ('cordova' in window) {

			$cordovaFacebook.api("me?fields=id,email,name,picture,gender,age_range,interested_in,birthday,work,education,bio,relationship_status,religion,political", [])
			.then(
				function(user) {
					console.log("getProfile", JSON.stringify(user));
					if (user.id) {
						user.authIdFacebook = user.id;
						delete user.id;
					}
					if (user.age_range && user.age_range.min < 21) {
						console.log("age_range < 21");
						deferred.reject("age_range");
						return;
					}
					if (data.accessToken) {
						user.accessTokenFacebook = data.accessToken;
					}
					deferred.resolve(user);
				},
				function(error) {
					console.log("getProfile error");
					deferred.reject(error);
				}
			);

		}
		else {
		
			hello('facebook').api('me', { "fields": "id,email,name,picture,gender,age_range,interested_in,birthday,work,education,bio" })
			.then(
				function(data) {
					console.log("getProfile", JSON.stringify(user));
					if (data.id) {
						data.authIdFacebook = data.id;
						delete data.id;
					}
					deferred.resolve(data);
				},
				function(error) {
					console.log("getProfile error");
					deferred.reject(error);
				}
			);
		}
		
		return deferred.promise;
	}

	
	function getPhotos(authIdFacebook) {
		var deferred = $q.defer();

		if ('cordova' in window) {
		
			$cordovaFacebook.api(authIdFacebook + "/photos")
			.then(
				function(data) {
					console.log("getPhotos", JSON.stringify(data));
					deferred.resolve(data.data);
				},
				function(error) {
					console.log("getPhotos error");
					deferred.reject(error);
				}
			);
		}
		else {
		
			hello('facebook').api(authIdFacebook + "/photos")
			.then(
				function(data) {
					console.log("getPhotos", JSON.stringify(data));
					deferred.resolve(data.data);
				},
				function(error) {
					console.log("getPhotos error");
					deferred.reject(error);
				}
			);
		}
		
		return deferred.promise;
	}
	
	
	function getFriends(data) {
		var deferred = $q.defer();

		if ('cordova' in window) {
		
			$cordovaFacebook.api(data.authIdFacebook + "/friends")
			.then(
				function(friends) {
					console.log("getFriends", JSON.stringify(friends));
					if (friends) {
						if (friends.summary && friends.summary.total_count)
							data.friendsFacebookTotal = friends.summary.total_count;
						if (friends.data)
							data.friendsFacebook = friends.data;
					}
					deferred.resolve(data);
				},
				function(error) {
					console.log("getFriends error");
					deferred.reject(data);
				}
			);
		}
		else {
		
			hello('facebook').api(data.authIdFacebook + "/friends")
			.then(
				function(friends) {
					console.log("getFriends", JSON.stringify(friends));
					if (friends) {
						data.friends = friends;
					}
					deferred.resolve(data);
				},
				function(error) {
					console.log("getFriends error");
					deferred.reject(error);
				}
			);
		}
		
		return deferred.promise;
	}


	function getAlbums(authIdFacebook) {
		var deferred = $q.defer();

		if ('cordova' in window) {
		
			$cordovaFacebook.api(authIdFacebook + "/albums")
			.then(
				function(data) {
					console.log("getAlbum");
					deferred.resolve(data.data);
				},
				function(error) {
					console.log("getAlbum error");
					deferred.reject(error);
				}
			);
		}
		else {
		
			hello('facebook').api(authIdFacebook + "/album")
			.then(
				function(data) {
					console.log("getAlbum");
					deferred.resolve(data.data);
				},
				function(error) {
					console.log("getAlbum error");
					deferred.reject(error);
				}
			);
		}
		
		return deferred.promise;
	}
	
	
	function getAlbumPhotos(albumId) {
		var deferred = $q.defer();

		if ('cordova' in window) {
		
			$cordovaFacebook.api(albumId + "/photos?fields=images")
			.then(
				function(data) {
					console.log("getAlbumPhotos");
					deferred.resolve(data.data);
				},
				function(error) {
					console.log("getAlbumPhotos error");
					deferred.reject(error);
				}
			);
		}
		else {
		
			hello('facebook').api(albumId + "/photos?fields=images")
			.then(
				function(data) {
					console.log("getAlbumPhotos");
					deferred.resolve(data.data);
				},
				function(error) {
					console.log("getAlbumPhotos error");
					deferred.reject(error);
				}
			);
		}
		
		return deferred.promise;
	}
	
	
	function getProfilePhotos(data) {
		var deferred = $q.defer();

		if (!data.authIdFacebook) {
			deferred.reject("fail to login");
			return;
		}
		
		getAlbums(data.authIdFacebook)
		.then(
			function(albums) {
				if (!albums || (albums && albums.length == 0)) {
					deferred.resolve(data);
					return;
				}
				
				var numAlbums = albums.length;
				var albumId = 0;
				var i = 0;
				var j = 0;
				
				for (i = 0; i < numAlbums; i++) {
					if (albums[i].name == "Profile Pictures") {
						albumId = albums[i].id;
					}
				}
				
				if (albumId == 0) {
					deferred.resolve(data);
					return;
				}
				
				getAlbumPhotos(albumId).then(
					function(photos) {
						var media = [];
						var url = "";
						var thumbnail = "";
						var numProfilePhotos = photos.length;
						
						if (numProfilePhotos == 1) {
							deferred.resolve(data);
							return;
						}
						
						for (i = 0; i < numProfilePhotos; i++) {
							url = "";
							thumbnail = "";
							for (j = 0; j < photos[i].images.length; j++) {
								if (photos[i].images[j].width == "640") {
									url = photos[i].images[j].source;
								}
								if (photos[i].images[j].width == "320") {
									thumbnail = photos[i].images[j].source;
								}
								if (url != "" && thumbnail != "")
									break;
							}
							if (url == "") {
								url = photos[i].images[0].source;
							}
							if (thumbnail == "") {
								thumbnail = photos[i].images[photos[i].images.length - 1].source;
							}
							media.push({ type: "i", url: url, thumbnail: thumbnail });
						}
						data.media = media;
						deferred.resolve(data);
					},
					function(error) {
						deferred.resolve(data);
					}
				)
			},
			function(error) {
				console.log("getProfilePhotos error");
				deferred.resolve(data);
			}
		);
		
		return deferred.promise;
	}
	

	return {
		login: login,
		logout: logout
	};
}]);
