angular.module('hh').service('MeetService', [
	'$q',
	'$http',
	'AppConfig',
	'UserService',
function(
	$q,
	$http,
	AppConfig,
	UserService
) {
	var userModel = UserService.getLoggedInUser();

	// api with callback
	this.callAPI = function(route, params, callback, isOpen) {
		var url = AppConfig.urlAPI + route + (isOpen ? '' : userModel.userId + '/' + userModel.session);
		$http.post(url, params)
		.success(function(data, status) {
			if (callback) {
				callback(data);
			}
		})
		.error(function(data, status) {
			console.log('Error:' + data + '\n Status:' + status);
		});
	};


	// api with promise
	function callAPIPromise(route, data) {
		var deferred = $q.defer();

		$http.post(AppConfig.urlAPI + route + userModel.userId + '/' + userModel.session, data)
		.success(function(res) {
			deferred.resolve(res);
		})
		.error(function(res) {
			deferred.reject(res);
		});
		return deferred.promise;
	};


	// getUsers
	this.getUsers = function(data) {
		return callAPIPromise('/users/get/', data);
	};


	// getMeet with meetId
	this.getMeet = function(data) {
		return callAPIPromise('/meet/get/', data);
	};


	// getMeet with meetId, userId, session
	this.getMyMeets = function(data) {
		return callAPIPromise('/mymeets/get/', data);
	};


	// getWhoLikesMe with userId, session
	this.getWhoLikesMe = function(data) {
		return callAPIPromise('/wholikesme/get/', data);
	};
	
	
	// getRecentLikes with userId, session
	this.getRecentLikes = function(data) {
		return callAPIPromise('/likes/recent/', data);
	};


	// add like with data = { userId: userId, likesUserId: likesUserId }
	this.addLike = function(data) {
		return callAPIPromise('/like/add/', data);
	};


	// accept like with data = { userId: userId, likesUserId: likesUserId }
	this.acceptLike = function(data) {
		return callAPIPromise('/like/accept/', data);
	};


	// delete like with data = { userId: userId, likesUserId: likesUserId }
	this.deleteLike = function(data) {
		return callAPIPromise('/like/delete/', data);
	};
	
	
	// rain check like with data = { userId: userId, likesUserId: likesUserId }
	this.rainCheckLike = function(data) {
		return callAPIPromise('/like/raincheck/', data);
	};
	

	// pass like with data = { userId: userId, likesUserId: likesUserId }
	this.passLike = function(data) {
		return callAPIPromise('/like/pass/', data);
	};
	
	
	// update likes with likeId, isViewed, userId, session
	this.updateLike = function(data) {
		return callAPIPromise('/likes/update/', data);
	};


	// send message to other user at meetId
	this.sendMessage = function(data) {
		return callAPIPromise('/message/send/', data);
	};


	// confirm meet
	this.confirmMeet = function(data) {
		return callAPIPromise('/meet/confirm/', data);
	};


	// checkin meet
	this.checkInMeet = function(data) {
		return callAPIPromise('/meet/checkin/', data);
	};


	// cancel meet
	this.cancelMeet = function(data) {
		return callAPIPromise('/meet/cancel/', data);
	};


	// done meet
	this.doneMeet = function(data) {
		return callAPIPromise('/meet/done/', data);
	};


	this.reviewUser = function(params, callback) {
		this.callAPI('/review/user/', params, callback);
	};


	this.reviewPlace = function(params, callback) {
		this.callAPI('/review/place/', params, callback);
	};
}
]);
