angular.module('hh')
.service('PhotoService',
[
	'$ionicLoading',
	'$q',
	'ImageResizer',
	'S3UploadService',
function(
	$ionicLoading,
	$q,
	ImageResizer,
	S3UploadService
) {
	function fullSize() {
		return { width: 640, height: 640 };
	}


	function thumbnailSize() {
		return { width: 320, height: 320 };
	}


	var defaultUploadCallbacks = {
		/*
		onProgress: function(e) { return e.lengthComputable && console.log('  Progress:', Math.round(e.loaded * 100 / e.total)) },
		*/
		onLoad: function(e) { return console.log('load', e.target.responseText, e) },
		onError: function(e) { return console.log('error', e) },
		onAbort: function(e) { return console.log('abort', e) },
	};


	function upload(url, image, options) {
		console.log('Uploading image to %s', url);
		S3UploadService.upload(url, image, angular.extend({}, defaultUploadCallbacks, options));
	}


	function generateThumbnailUrl(url) {
		var parts = url.split('.');
		parts.splice(parts.length - 1, 0, 't');
		return parts.join('.');
	}


	function uploadImages(scope, mediaType) {
		var images = Array.prototype.slice.call(arguments, 2);

		$ionicLoading.show()

		$q.all(images).then(function(images) {
			var full = images[0];
			var thumb = images[1];

			S3UploadService.generateUrl(mediaType)
				.then(_upload, logError);

			function _upload(url) {
				var urlThumbnail = generateThumbnailUrl(url);

				// upload thumbnail
				upload(urlThumbnail, thumb);
				console.log('urlThumbnail: ' + urlThumbnail);

				// upload file
				upload(url, full, {
					onLoad: function() {
						console.log('url: ' + url);
						scope.addMedia({
							type: 'i',
							url: url,
							thumbnail: urlThumbnail
						});
					}
				});
			}
		});
	}


	function resize(image, coords, fn) {
		return ImageResizer.resize(image, coords).then(function(src) {
			return ImageResizer.crop(src, coords).then(function(src) {
				if (typeof fn === 'function') fn(src);
				return src;
			});
		});
	}


	function logError(arguments) {
		console.log('TODO: error', arguments);
		$ionicLoading.hide();
	}
 
 
 	function open(scope, isAlbum) {
		navigator.camera.getPicture(
			function(image) {
				console.log("PhotoService image resize");

				var mediaType = 'image/jpg';
				image = 'data:' + mediaType + ';base64,' + image;

				var full = image;
				var thumb = resize(image, thumbnailSize());

				console.log("PhotoService uploadImage");
				
				uploadImages(scope, mediaType, full, thumb);
			},
			function(err) {
				// fallback to selecting from the Photo Gallery.
				// the onchange handler will automatically trigger
				console.log(err);
			},
			{
				allowEdit: true,
				quality: 60,
				targetWidth: fullSize().width,
				targetHeight: fullSize().height,
				cameraDirection: navigator.camera.Direction.FRONT,
				sourceType: isAlbum ? navigator.camera.PictureSourceType.PHOTOLIBRARY : navigator.camera.PictureSourceType.CAMERA,
				destinationType: navigator.camera.DestinationType.DATA_URL,
				encodingType: navigator.camera.EncodingType.JPG
			}
		);
	}

	
	return {
		open: open,
	};
}]);
