'use strict';

var _isInitPushEvents = false;

angular.module('hh').factory('PushService', [
	'$rootScope',
	'$state',
	'$q',
	'$http',
	'$timeout',
	'$ionicPlatform',
	'$cordovaPushV5',
	'AppConfig',
	'UserService',
function(
	$rootScope,
	$state,
	$q,
	$http,
	$timeout,
	$ionicPlatform,
	$cordovaPushV5,
	AppConfig,
	UserService
) {

	function init() {

		$cordovaPushV5.unregister();

		var deferred = $q.defer();

		if (!_isInitPushEvents) {
			_isInitPushEvents = true;
			initPushEvents();
		}
											 
		$cordovaPushV5.initialize(AppConfig.push)
		.then(
			function(data) {
				console.log("PushService register");
				$cordovaPushV5.onNotification();
				$cordovaPushV5.onError();
				$cordovaPushV5.register(AppConfig.push)
				.then(
					function(deviceToken) {
						console.log("PushService deviceToken: " + deviceToken);
						var userModel = UserService.getLoggedInUser();
						if (userModel) {
							$http.post(AppConfig.urlAPI + "/push/add/" + userModel.userId + "/" + userModel.session, { pushType: Object.keys(AppConfig.push)[0], pushToken: deviceToken });
						}
						$cordovaPushV5.setBadgeNumber(0);
						deferred.resolve();
					},
					function(error) {
						console.log("PushService error: " + JSON.stringify(error));
						deferred.reject();
					}
				);
			},
			function(error) {
				console.log("PushService initialize error", JSON.stringify(error));
				deferred.reject();
			}
		);
		
		return deferred.promise;
	}
	

	function initPushEvents() {
	
		$rootScope.$on('$cordovaPushV5:errorOcurred', function(event, error){
    		console.log("PushService onError: " + error.message);
		});
	
		$rootScope.$on('$cordovaPushV5:notificationReceived', function(event, notification) {

			console.log("PushService onPushNotification: " + JSON.stringify(notification));

			//alert("PushService onPushNotification: " + JSON.stringify(notification));

			if (ionic.Platform.isIOS())
				$cordovaPushV5.setBadgeNumber(0);
			
			if (notification.additionalData && notification.additionalData.link) {
				console.log("PushService link", notification.additionalData.link);
				_urlScheme = notification.additionalData.link;
				var params = getParamsObjFromUrl(_urlScheme);

				if (notification.sound) {
					params.sound = notification.sound;
				}

				if (notification.message) {
					params.text = notification.message;
				}

				// show would like to meet push
				if (notification.additionalData.link.indexOf("jones://like") != -1) {
					if (params.likeUserId && params.eventId) {
						$rootScope.$emit("invitePush", params);
					}
				}
				else {
					$rootScope.$emit("messagePush", params);
					$rootScope.$emit("checkMeet");
				}
			}
		});
	}


	return {
		init: init
	};
}]);
