// 	http://tryjones.my/api/json/file/png/new
angular.module('hh').service('S3UploadService',
[
	'$http',
	'AppConfig',
function(
	$http,
	AppConfig
) {
    var awsKey = AppConfig.S3.key,
        policyBase64 = AppConfig.S3.policy,
        signature = AppConfig.S3.signature;

	function _getFormData(key, file) {
		var fd = new FormData();
		if (typeof file === 'string') {
			file = fileFromBase64String(file);
		}
		var mediaType = file.type;

		fd.append('key', key);
		fd.append('acl', 'public-read');
		fd.append('success_action_status', 201);
		fd.append('Content-Type', mediaType);
		fd.append('AWSAccessKeyId', awsKey);
		fd.append('policy', policyBase64);
		fd.append('signature', signature);
		fd.append('file', file);

		return fd;
	}

	function upload(url, file, options) {
		var parts = url.split(/.com\//);
		var uploadUrl = parts[0] + '.com/';
		var key = parts[1];

		var formData = _getFormData(key, file);
		// TODO: switch this to $http to have the $digest cycle
		var xhr = new XMLHttpRequest();

		xhr.upload.addEventListener('progress', options.onProgress || angular.noop, false);
		xhr.addEventListener('load', options.onLoad || angular.noop, false);
		xhr.addEventListener('error', options.onError || angular.noop, false);
		xhr.addEventListener('abort', options.onAbort || angular.noop, false);

		xhr.open('POST', uploadUrl, true);

		xhr.send(formData);
	}

	function generateUrl(mediaType) {
		return $http.get(AppConfig.urlAPI + '/file/' + mediaType + '/new')
			.then(function(response) {
				return response.data.data.url;
			});
	}

	function contentTypeFromBase64String(str) {
		// base64 string follows the following format.
		// data:[<mediatype>][;base64],<data>
		// default based on https://developer.mozilla.org/en-US/docs/Web/HTTP/data_URIs
		return str.slice(5).split(',')[0].split(';')[0] || 'text/plain;charset=US-ASCII';
	}

	// http://stackoverflow.com/questions/16245767/creating-a-blob-from-a-base64-string-in-javascript
	function fileFromBase64String(base64str, contentType, sliceSize) {
		sliceSize = sliceSize || 512;

		// default content-type
		if (typeof contentType === 'undefined') {
			if (base64str.slice(0, 4) === 'data') {
				contentType = contentTypeFromBase64String(base64str);
				base64str = base64str.replace(/^data:[^,]+,/, '');
			}
			else contentTypeFromBase64String = '';
		}

		var byteCharacters = atob(base64str);
		var byteArrays = [];

		for (var offset = 0; offset < byteCharacters.length; offset += sliceSize) {
			var slice = byteCharacters.slice(offset, offset + sliceSize);

			var byteNumbers = new Array(slice.length);
			for (var i = 0; i < slice.length; i++) {
				byteNumbers[i] = slice.charCodeAt(i);
			}

			var byteArray = new Uint8Array(byteNumbers);

			byteArrays.push(byteArray);
		}

		var blob = new Blob(byteArrays, {type: contentType});
		return blob;
	}

	function mediaTypeFromBase64String(str) {
		return /^data:([^;]*)/.match(str)[0];
	}
 
	function uploadFile(fileURL, fileDevicePath, mediaType, onSuccess, onError) {
	
		console.log('uploadFile');

 		var parts = fileURL.split(/.com\//);
		var uploadUrl = parts[0] + '.com/';
		var key = parts[1];
		var fileName = key.substring(2);
		
		console.log(uploadUrl, key, fileName);
						
		var ft = new FileTransfer(),
			options = new FileUploadOptions();
 
		options.fileKey = 'file';
		options.fileName = fileName;
		options.mimeType = mediaType;
		options.chunkedMode = false;
		options.params = {
			'key': key,
			'AWSAccessKeyId': awsKey,
			'success_action_status': '201',
			'acl': 'public-read',
			'policy': policyBase64,
			'signature': signature,
			'Content-Type': mediaType
        };
 
		ft.upload(fileDevicePath, uploadUrl, onSuccess, onError, options);
	}

	return {
		upload: upload,
		uploadFile: uploadFile,
		generateUrl: generateUrl
	};

}]);
