angular.module('hh')
.service('VideoService',
[
	'$ionicLoading',
	'$timeout',
	'$q',
	'S3UploadService',
function(
	$ionicLoading,
	$timeout,
	$q,
	S3UploadService
) {
	function fullSize() {
		return { width: 640, height: 640 };
	}


	function thumbnailSize() {
		return { width: 320, height: 320 };
	}


	function generateThumbnailUrl(url) {
		var parts = url.split('.');
		parts.splice(parts.length - 1, 1, 't.jpg');
		return parts.join('.');
	}


	function uploadFiles(scope, mediaTypes, filePaths) {
		S3UploadService.generateUrl(mediaTypes[0])
		.then(_upload, logError);

		function _upload(url) {
			var urlThumbnail = generateThumbnailUrl(url);

			// upload thumbnail
			S3UploadService.uploadFile(urlThumbnail, filePaths[1], mediaTypes[1],
				function(result) {
					console.log('urlThumbnail: ' + urlThumbnail);
					
					// clean up
					deleteFile(filePaths[0]);
				},
				function(error) {
					$ionicLoading.hide();
					console.log('VideoService upload thumbnail error');
				}
			);

			// upload video file
			S3UploadService.uploadFile(url, filePaths[0], mediaTypes[0],
				function(result) {
					console.log('url: ' + url);
					scope.addMedia({
						type: 'v',
						url: url,
						thumbnail: urlThumbnail
					});

					// clean up
					deleteFile(filePaths[1]);
				},
				function(error) {
					$ionicLoading.hide();
					console.log('VideoService upload file error');
				}
			);
		}
	}

	
	function logError(arguments) {
		$ionicLoading.hide();
		console.log('VideoService logError', arguments);
	}
	

 	function open(scope) {
		navigator.device.capture.captureVideo(
			function(mediaFiles) {
				$timeout(function() { transcodeVideo(scope, mediaFiles); }, 100);
			},
			function(error) {
				console.log('VideoService videoCapture error: ' + error);
			},
			{
				limit: 1,
				duration: 15
			}
		);
	}
	
	
	function transcodeVideo(scope, mediaFiles) {
		$ionicLoading.show();

		var file = mediaFiles[0];
		var videoFileName = Date.now() + '.mp4'; // I suggest a uuid

		VideoEditor.transcodeVideo(
			function(videoFilePath) {
				// videoPath is the path to the transcoded video on the device
				console.log('videoService videoTranscodeSuccess: ' + videoFilePath);
				// generate thumbnail
				VideoEditor.createThumbnail(
					function(thumbnailFilePath) {
						uploadFiles(scope, ['video/mp4', 'image/jpeg'], [videoFilePath, thumbnailFilePath]);
					},
					function(error) {
						$ionicLoading.hide();
						console.log('VideoService createThumbnail error: ' + error);
					},
					{
						fileUri: videoFilePath, // the path to the video on the device
						outputFileName: Date.now() + '.t.jpg', // the file name for the JPEG image
						atTime: 0, // optional, location in the video to create the thumbnail (in seconds)
						width: thumbnailSize().width, // optional, width of the thumbnail
						height: thumbnailSize().height, // optional, height of the thumbnail
						quality: 80 // optional, quality of the thumbnail (between 1 and 100)
					}
				);
			},
			function(error) {
				$ionicLoading.hide();
				console.log('VideoService transcodeVideo error: ' + error);
			},
			{
				fileUri: file.fullPath,
				outputFileName: videoFileName,
				outputFileType: VideoEditorOptions.OutputFileType.MPEG4,
				optimizeForNetworkUse: VideoEditorOptions.OptimizeForNetworkUse.YES,
				saveToLibrary: false,
				maintainAspectRatio: true,
				width: fullSize().width,
				height: fullSize().height,
				videoBitrate: 1000000, // 1 megabit
				audioChannels: 1,
				audioSampleRate: 44100,
				audioBitrate: 128000, // 128 kilobits
				progress: function(info) {
					console.log('videoService transcodeVideo progress: ' + info);
				}
			}
		);
	}
	
	
	function deleteFile(filePath) {
		// TODO: Android
		var path = filePath.substring(0, filePath.lastIndexOf('/'));
		var filename = filePath.split('/').pop();

		window.resolveLocalFileSystemURL(path, function(dir) {
			dir.getFile(filename, { create: false },
				function(fileEntry) {
					fileEntry.remove(
						function() {
							// file deleted
							console.log("file deleted");
						},
						function(error) {
							// error
							console.log(error);
						},
						function(){
							// file doesn't exist
							console.log("file doesn't exist");
						}
					);
				}
			);
		});
	}
	
	
	return {
		open: open,
	};
}]);
